package com.starbuds.server.service.activity.grab;

import cn.hutool.core.convert.Convert;
import com.alibaba.fastjson.JSON;
import com.starbuds.server.common.constant.activity.ActivityGrabPeriodStatusEnum;
import com.starbuds.server.common.constant.cache.RedisActivityKeys;
import com.starbuds.server.common.constant.cache.RedisRtcKeys;
import com.starbuds.server.common.constant.common.GlobalApiError;
import com.starbuds.server.common.constant.common.TimeConstants;
import com.starbuds.server.common.constant.message.im.ImTargetType;
import com.starbuds.server.common.pojo.activity.ActivityGrabPeriodVO;
import com.starbuds.server.common.pojo.api.Result;
import com.starbuds.server.common.pojo.db.ActivityGrabConfigDO;
import com.starbuds.server.common.pojo.db.ActivityGrabPeriodDO;
import com.starbuds.server.common.pojo.db.ActivityGrabPeriodItemDO;
import com.starbuds.server.common.pojo.message.im.activity.GrabPlayerNumChangeMsg;
import com.starbuds.server.common.service.data.RedisService;
import com.starbuds.server.common.service.finance.WalletService;
import com.starbuds.server.common.service.message.ImService;
import com.starbuds.server.service.activity.dao.ActivityGrabPeriodDAO;
import com.starbuds.server.service.activity.dao.ActivityGrabPeriodItemDAO;
import lombok.extern.slf4j.Slf4j;
import org.apache.dubbo.config.annotation.Reference;
import org.springframework.beans.BeanUtils;
import org.springframework.stereotype.Component;

import javax.annotation.Resource;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.Optional;
import java.util.concurrent.TimeUnit;

/**
 * 枪龙活动服务
 *
 * @author Lay
 * @date 2021/3/29
 */
@Component
@Slf4j
public class GrabPeriodProvider {

    @Resource
    ActivityGrabPeriodDAO activityGrabPeriodDAO;

    @Resource
    ActivityGrabPeriodItemDAO activityGrabPeriodItemDAO;

    @Resource
    GrabSettleProvider grabSettleProvider;

    @Reference(version = "1.0.0")
    RedisService redisService;

    @Reference(version = "1.0.0")
    WalletService walletService;

    @Reference(version = "1.0.0")
    ImService imService;

    public ActivityGrabPeriodVO getPeriod(long activityId) {
        // 拉取最新一期活动
        ActivityGrabPeriodDO periodDO = activityGrabPeriodDAO.selectLatest(activityId);
        if (periodDO != null) {
            ActivityGrabPeriodVO periodVO = new ActivityGrabPeriodVO();
            BeanUtils.copyProperties(periodDO, periodVO);
            ActivityGrabConfigDO configDO = JSON.parseObject(periodDO.getConfigSnapshot(), ActivityGrabConfigDO.class);
            periodVO.setPrice(configDO.getPrice());
            periodVO.setPriceType(configDO.getPriceType());
            periodVO.setMaxPlayerNum(configDO.getPlayerNum());
            periodVO.setCurPlayerNum(activityGrabPeriodItemDAO.countPlayers(periodDO.getPeriodId()));
            periodVO.setItemsCount(configDO.getItemsCount());
            return periodVO;
        }
        return null;
    }

    public Long join(long periodId, long userId) {
        Long balance = null;

        //获取活动
        ActivityGrabPeriodDO periodDO = activityGrabPeriodDAO.selectById(periodId);
        if (periodDO != null) {
            // 判断状态
            if (periodDO.getPeriodStatus() == ActivityGrabPeriodStatusEnum.Going.getValue()) {
                ActivityGrabConfigDO configDO = JSON.parseObject(periodDO.getConfigSnapshot(), ActivityGrabConfigDO.class);

                long blanc = walletService.getBalance(userId, configDO.getPriceType());
                if (blanc < configDO.getPrice()) {
                    log.info("参与失败余额不足" + periodId + "|" + userId);
                    return null;
                }

                //分配位置
                int seat = -1;
                for (int i = 0; i < configDO.getPlayerNum(); i++) {
                    //位置
                    if (touchAtivityPeriodPosition(periodId, i, userId, true)) {
                        seat = i;
                        break;
                    }
                }
                //获取到位置
                if (seat >= 0) {
                    // 扣费
                    balance = walletService.reduceCoin(userId, configDO.getPriceType(), configDO.getPrice(), "预购金冻结", periodId);

                    if (balance == null || balance < 0) {
                        log.info("参与失败金额不足错误：" + periodId + "|" + userId);
                        // 返还位置
                        redisService.hashDel(RedisActivityKeys.getActivityPeriodidSeat(periodId), String.valueOf(seat));
                        return balance;
                    }

                    try {
                        // 创建参与记录
                        ActivityGrabPeriodItemDO periodItemDO = new ActivityGrabPeriodItemDO();
                        periodItemDO.setUserId(userId);
                        periodItemDO.setPeriodId(periodId);
                        periodItemDO.setJoinTime(System.currentTimeMillis());
                        activityGrabPeriodItemDAO.insert(periodItemDO);
                    } catch (Exception e) {
                        log.error("创建参与记录失败", e);
                        // 返还金额 减少参与人数
                        walletService.addCoin(userId, configDO.getPriceType(), configDO.getPrice(), "预购金返还", periodId);
                        redisService.hashDel(RedisActivityKeys.getActivityPeriodidSeat(periodId), String.valueOf(seat));
                    }

                    int playerNum = getPeriodIdUserMap(periodId).size();

                    boolean settle = playerNum == configDO.getPlayerNum();
                    // 发送成员加入消息
                    GrabPlayerNumChangeMsg playerNumChangeMsg = new GrabPlayerNumChangeMsg();
                    playerNumChangeMsg.setPlayerNum(playerNum);
                    playerNumChangeMsg.setPeriodId(periodId);
                    playerNumChangeMsg.setSettle(settle);
                    imService.sendRtcRoomMsg(ImTargetType.GROUP, null, null, -1001L, playerNumChangeMsg);

                    // 异步结算判定
                    if (settle) {
                        //开奖加锁防止并发
                        String lockKey = RedisActivityKeys.getActivityPeriodidLock(periodId);
                        if (redisService.setIfAbsent(lockKey, String.valueOf(userId), TimeConstants.SECOND_30_MILLS, TimeUnit.MILLISECONDS)) {
                            List<Long> userIds = activityGrabPeriodItemDAO.selectUserIds(periodId);
                            grabSettleProvider.settleAsync(periodId, userIds);
                        }
                    }
                } else {
                    //没有位置可用
                    int playerNum = getPeriodIdUserMap(periodId).size();
                    log.warn("人数异常pid{},userid{},set{}, playerNum {} maxPlayerNum {}", periodId, userId, seat, playerNum, configDO.getPlayerNum());
                    return -1L;
                }
            } else {
                log.warn("状态异常 {}", periodDO.getPeriodStatus());
            }
        } else {
            log.warn("未找到活动期数 {}", periodId);
        }
        return balance;
    }


//    public Long joinOlder(long periodId, long userId) {
//        Long balance = null;
//        // 判断人数
//        ActivityGrabPeriodDO periodDO = activityGrabPeriodDAO.selectById(periodId);
//        if (periodDO != null) {
//            // 判断状态
//            if (periodDO.getPeriodStatus() == ActivityGrabPeriodStatusEnum.Going.getValue()) {
//                ActivityGrabConfigDO configDO = JSON.parseObject(periodDO.getConfigSnapshot(), ActivityGrabConfigDO.class);
//                // 先判断一次人数 减少冲突
//                if (Optional.ofNullable(redisService.hashGet(RedisActivityKeys.ACTIVITY_GRAB_PERIOD_NUMS, String.valueOf(periodId))).map(Convert::toLong).orElse(0L) >= configDO.getPlayerNum()) {
//                    return balance;
//                }
//
//                // 人数确定
//                Integer playerNum = Math.toIntExact(redisService.hashIncreaseLong(RedisActivityKeys.ACTIVITY_GRAB_PERIOD_NUMS, String.valueOf(periodId), 1));
//                if (playerNum <= configDO.getPlayerNum()) {
//                    // 扣费
//                    balance = walletService.reduceCoin(userId, configDO.getPriceType(), configDO.getPrice(), "预购金冻结", periodId);
//
//                    if (balance == null || balance < 0) {
//                        // 返还人数
//                        redisService.hashIncreaseLong(RedisActivityKeys.ACTIVITY_GRAB_PERIOD_NUMS, String.valueOf(periodId), -1);
//                        return balance;
//                    }
//
//                    try {
//                        // 创建参与记录
//                        ActivityGrabPeriodItemDO periodItemDO = new ActivityGrabPeriodItemDO();
//                        periodItemDO.setUserId(userId);
//                        periodItemDO.setPeriodId(periodId);
//                        periodItemDO.setJoinTime(System.currentTimeMillis());
//                        activityGrabPeriodItemDAO.insert(periodItemDO);
//                    } catch (Exception e) {
//                        log.error("创建参与记录失败", e);
//                        // 返还金额 减少参与人数
//                        walletService.addCoin(userId, configDO.getPriceType(), configDO.getPrice(), "预购金返还", periodId);
//                        redisService.hashIncreaseLong(RedisActivityKeys.ACTIVITY_GRAB_PERIOD_NUMS, String.valueOf(periodId), -1);
//                    }
//
//                    boolean settle = playerNum.equals(configDO.getPlayerNum());
//                    // 发送成员加入消息
//                    GrabPlayerNumChangeMsg playerNumChangeMsg = new GrabPlayerNumChangeMsg();
//                    playerNumChangeMsg.setPlayerNum(playerNum);
//                    playerNumChangeMsg.setPeriodId(periodId);
//                    playerNumChangeMsg.setSettle(settle);
//                    imService.sendRtcRoomMsg(ImTargetType.GROUP, null, null, -1001L, playerNumChangeMsg);
//
//                    // 异步结算判定
//                    if (settle) {
//                        List<Long> userIds = activityGrabPeriodItemDAO.selectUserIds(periodId);
//                        grabSettleProvider.settleAsync(periodId, userIds);
//                    }
//                } else {
//                    // 返还人数
//                    redisService.hashIncreaseLong(RedisActivityKeys.ACTIVITY_GRAB_PERIOD_NUMS, String.valueOf(periodId), -1);
//                    log.warn("人数异常 playerNum {} maxPlayerNum {}", playerNum, configDO.getPlayerNum());
//                }
//            } else {
//                log.warn("状态异常 {}", periodDO.getPeriodStatus());
//            }
//        } else {
//            log.warn("未找到活动期数 {}", periodId);
//        }
//        return balance;
//    }


    /**
     * 抢活动位置
     *
     * @param periodId
     * @param posion
     * @param userId
     * @param skipLock
     * @return
     */
    private boolean touchAtivityPeriodPosition(Long periodId, Integer posion, Long userId, boolean skipLock) {

        // 判断用户是否在活动位置
        if (getPeriodUserSeatNo(periodId, userId) > -1) {
            return false;
        }

        return redisService.hashPutIfAbsent(RedisActivityKeys.getActivityPeriodidSeat(periodId), String.valueOf(posion), String.valueOf(userId));
    }


    /**
     * 判断用户是否在位置上
     *
     * @param periodId
     * @param userId
     * @return
     */
    public int getPeriodUserSeatNo(Long periodId, Long userId) {
        Map<Integer, Long> seatUserMap = getPeriodIdUserMap(periodId);
        for (Map.Entry<Integer, Long> entry : seatUserMap.entrySet()) {
            if (userId.equals(entry.getValue())) {
                return entry.getKey();
            }
        }
        return -1;
    }

    /**
     * 获取活动期数用户
     *
     * @param periodId
     * @return seatNo:userId
     */
    public Map<Integer, Long> getPeriodIdUserMap(Long periodId) {
        Map<Object, Object> map = redisService.hashGetAll(RedisActivityKeys.getActivityPeriodidSeat(periodId));
        Map<Integer, Long> seatUserIds = new HashMap<>();
        for (Map.Entry<Object, Object> entry : map.entrySet()) {
            seatUserIds.put(Convert.toInt(entry.getKey()), Convert.toLong(entry.getValue()));
        }
        return seatUserIds;
    }


}
