package com.starbuds.server.service.activity.impl;

import cn.hutool.core.collection.CollectionUtil;
import com.starbuds.server.common.constant.message.im.ImTargetType;
import com.starbuds.server.common.pojo.db.ActivityBarrageDO;
import com.starbuds.server.common.pojo.db.ActivityBroadcastHisDO;
import com.starbuds.server.common.pojo.message.im.rtc.RtcActivityRankMsg;
import com.starbuds.server.common.pojo.user.UserBasicInfo;
import com.starbuds.server.common.service.activity.ActivityBarrageService;
import com.starbuds.server.common.service.core.IdentityService;
import com.starbuds.server.common.service.message.ImService;
import com.starbuds.server.common.service.user.UserService;
import com.starbuds.server.common.utils.DateUtil;
import com.starbuds.server.service.activity.dao.ActivityBarrageDAO;
import com.starbuds.server.service.activity.dao.ActivityBroadcastHisDAO;
import lombok.extern.slf4j.Slf4j;
import org.apache.dubbo.config.annotation.Reference;
import org.apache.dubbo.config.annotation.Service;

import javax.annotation.Resource;
import java.util.*;

/**
 * 弹幕实现
 *
 * @Author: lujiao
 * @Date: 2021/8/9 14:26
 */
@Slf4j
@Service(version = "1.0.0")
public class ActivityBarrageServiceImpl implements ActivityBarrageService {

    @Resource
    private ActivityBarrageDAO activityBarrageDAO;

    @Resource
    private ActivityBroadcastHisDAO broadcastHisDAO;

    @Reference(version = "1.0.0")
    private IdentityService identityService;

    @Reference(version = "1.0.0")
    private UserService userService;

    @Reference(version = "1.0.0")
    private ImService imService;

    private static final Long[] LEVEL_JUDGE = new Long[]{188000L, 334400L, 520000L};

    /**
     * 发送弹幕
     *
     * @param userId
     * @param context
     * @param activityId
     */
    @Override
    public void sendBarrage(Long userId, String context, Long activityId) {
        ActivityBarrageDO barrageDO = new ActivityBarrageDO();
        barrageDO.setActivityBarrageId(identityService.getId());
        barrageDO.setActivityId(activityId);
        barrageDO.setUserId(userId);
        barrageDO.setContext(context);
        barrageDO.setCreateDate(DateUtil.getIntDate());
        barrageDO.setCreateTime(System.currentTimeMillis());
        activityBarrageDAO.insert(barrageDO);
    }


    /**
     * 获取弹幕列表
     *
     * @param userId
     * @return
     */
    @Override
    public List<ActivityBarrageDO> getBarrageList(Long userId, Long activityId) {
        Long yesterday = System.currentTimeMillis() - DateUtil.DAY_OF_MILLION;
        List<ActivityBarrageDO> lastList = activityBarrageDAO.getBarrageList(activityId, yesterday);
        if (CollectionUtil.isNotEmpty(lastList)) {
            long hasSelf = lastList.stream().filter(i -> i.getUserId().equals(userId)).count();
            if (hasSelf == 0) {
                // 如果没有, 说明需要在顶部增加一个
                ActivityBarrageDO selfBarrage = getLastBarrage(userId, activityId);
                if (selfBarrage != null) {
                    lastList.add(0, selfBarrage);
                }
            }
            return lastList;
        }
        // 把用户自己的列在第一位上
        return new ArrayList<>();
    }


    /**
     * 获取最新的一条弹幕
     *
     * @param userId
     * @param activityId
     * @return
     */
    @Override
    public ActivityBarrageDO getLastBarrage(Long userId, Long activityId) {
        return activityBarrageDAO.getLastBarrage(userId, activityId);
    }

    /**
     * 发送全局广播
     */
    @Override
    public void sendActivityRankBroadcast(Long sendUserId, Long receiveUserId, Long giftRank, Long activityId) {
        List<Long> needToBroadcast = buildBroadcastList(sendUserId, receiveUserId, giftRank, activityId);
        Set<Long> userIds = new HashSet<>();
        userIds.add(sendUserId);
        userIds.add(receiveUserId);
        Map<Long, UserBasicInfo> userBasicInfoMap = userService.getBasicInfoMap(userIds);
        UserBasicInfo sendUser = userBasicInfoMap.get(sendUserId);
        UserBasicInfo receiver = userBasicInfoMap.get(receiveUserId);
        needToBroadcast.forEach(i -> {
            doSendActivityRankBroadcast(sendUser, receiver, i, activityId);
        });
    }

    private List<Long> buildBroadcastList(Long sendUserId, Long receiveUserId, Long giftRank, Long activityId) {
        List<Long> res = new ArrayList<>();
        for (int i = 0; i < LEVEL_JUDGE.length; i++) {
            if (giftRank >= LEVEL_JUDGE[i]) {
                res.add(LEVEL_JUDGE[i]);
            }
        }
        int sendHisCount = broadcastHisDAO.getSendBroadcastCount(sendUserId, receiveUserId, activityId);
        int begin = 0;
        Iterator<Long> it = res.iterator();
        if (sendHisCount > res.size()) {
            // 特殊处理, 防止报错
            sendHisCount = res.size();
        }
        while (it.hasNext()) {
            if (begin < sendHisCount) {
                Long tmp = it.next();
                it.remove();
                begin++;
            } else {
                break;
            }
        }
        return res;
    }

    /**
     * 真正发送的方法
     *
     * @param sendUser
     * @param receiveUser
     * @param giftRank
     */
    public void doSendActivityRankBroadcast(UserBasicInfo sendUser, UserBasicInfo receiveUser, Long giftRank, Long activityId) {
        if (sendUser == null || receiveUser == null) {
            return;
        }
        // 插入记录
        ActivityBroadcastHisDO po = new ActivityBroadcastHisDO();
        po.setActivityId(activityId);
        po.setCreateTime(System.currentTimeMillis());
        po.setGiftRank(giftRank);
        po.setSendUserId(sendUser.getUserId());
        po.setReceiveUserId(receiveUser.getUserId());
        po.setHisId(identityService.getId());
        broadcastHisDAO.insert(po);
        RtcActivityRankMsg rankMsg = new RtcActivityRankMsg();
        rankMsg.setSendUserId(sendUser.getUserId());
        rankMsg.setSendUserName(sendUser.getUserName());
        rankMsg.setGiftRank(giftRank);
        rankMsg.setReceiveUserId(receiveUser.getUserId());
        rankMsg.setReceiveUserName(receiveUser.getUserName());
        // 发送全局广播
        imService.sendRtcRoomMsg(ImTargetType.ALL, null, null, null, rankMsg);
    }

    /**
     * 测试用发送, 调试后删除
     *
     * @param sendUserId
     * @param receiveUserId
     * @param giftRank
     */
    @Override
    public void sendActivityRankBroadcastTest(Long sendUserId, Long receiveUserId, Long giftRank) {
        Set<Long> userIds = new HashSet<>();
        userIds.add(sendUserId);
        userIds.add(receiveUserId);
        Map<Long, UserBasicInfo> userBasicInfoMap = userService.getBasicInfoMap(userIds);
        UserBasicInfo sendUser = userBasicInfoMap.get(sendUserId);
        UserBasicInfo receiver = userBasicInfoMap.get(receiveUserId);
        doSendActivityRankBroadcast(sendUser, receiver, giftRank, 10L);
    }
}
