package com.starbuds.server.service.activity.impl;

import cn.hutool.core.collection.CollectionUtil;
import cn.hutool.core.convert.Convert;
import com.alibaba.fastjson.JSON;
import com.starbuds.server.common.pojo.activity.GiftItem;
import com.starbuds.server.common.pojo.activity.RewardItem;
import com.starbuds.server.common.pojo.api.Result;
import com.starbuds.server.common.pojo.db.*;
import com.starbuds.server.common.pojo.finance.BagItem;
import com.starbuds.server.common.pojo.user.GiftWallItem;
import com.starbuds.server.common.service.activity.ActivityFestivalService;
import com.starbuds.server.common.service.core.IdentityService;
import com.starbuds.server.common.service.finance.BagService;
import com.starbuds.server.common.service.user.UserGiftWallService;
import com.starbuds.server.common.utils.DateUtil;
import com.starbuds.server.service.activity.dao.*;
import lombok.extern.slf4j.Slf4j;
import org.apache.dubbo.config.annotation.Method;
import org.apache.dubbo.config.annotation.Reference;
import org.apache.dubbo.config.annotation.Service;
import org.springframework.transaction.annotation.Transactional;

import javax.annotation.Resource;
import java.util.*;
import java.util.stream.Collectors;

/**
 * @Author: lujiao
 * @Date: 2021/9/3 15:13
 */
@Slf4j
@Service(version = "1.0.0")
public class ActivityFestivalServiceImpl implements ActivityFestivalService {

    @Resource
    private ActivityDAO activityDAO;

    @Resource
    private ActivityCollectDAO activityCollectDAO;

    @Resource
    private ActivityLotteryDAO activityLotteryDAO;

    @Resource
    private UserBagRewardCostHisDAO userBagRewardCostHisDAO;

    @Reference(version = "1.0.0")
    private UserGiftWallService userGiftWallService;

    @Reference(version = "1.0.0")
    private BagService bagService;

    @Reference(version = "1.0.0")
    private IdentityService identityService;

    @Resource
    private ActivityLotteryHisDAO activityLotteryHisDAO;

    /**
     * 获取活动列表
     *
     * @param activityIds
     * @return
     */
    @Override
    public List<ActivityDO> getActivityList(List<Long> activityIds) {
        return activityDAO.getActivityList(activityIds);
    }


    /**
     * 获取活动列表
     *
     * @param collectActivityIds
     * @return
     */
    @Override
    public List<ActivityCollectDO> getActivityCollectList(List<Long> collectActivityIds) {
        return activityCollectDAO.getActivityCollectList(collectActivityIds);
    }

    /**
     * 获取活动列表
     *
     * @param lotteryActivityIds
     * @return
     */
    @Override
    public List<ActivityLotteryDO> getActivityLotteryList(List<Long> lotteryActivityIds) {
        return activityLotteryDAO.getActivityLotteryList(lotteryActivityIds);
    }

    /**
     * 兑换奖品
     *
     * @param collectId
     * @param userId
     * @return
     */
    @Override
    public Result receiveActivityReward(String collectId, Long userId, ActivityDO activityDO) {
        // 检测之前获取的
        ActivityCollectDO collectDO = activityCollectDAO.selectById(collectId);
        if (collectDO == null) {
            return Result.ofOperationFailed("未找到活动");
        }
        List<GiftItem> itemList = JSON.parseArray(collectDO.getCollectGifts(), GiftItem.class);
        if (CollectionUtil.isEmpty(itemList)) {
            return Result.ofOperationFailed("未配置收集的礼物");
        }
        List<Long> giftIds = itemList.stream().map(GiftItem::getGiftId).collect(Collectors.toList());
        Map<Long, Integer> wallGiftNumMap = getWallGiftNumMap(giftIds, userId, activityDO, collectId);
        if (CollectionUtil.isEmpty(wallGiftNumMap)) {
            return Result.ofOperationFailed("未收到活动的礼物");
        }
        boolean isMatch = true;
        for (GiftItem item : itemList) {
            // 未达标
            Integer tmp = wallGiftNumMap.get(item.getGiftId());
            if (tmp == null) {
                tmp = 0;
            }
            if (tmp < item.getQuantity()) {
                isMatch = false;
                break;
            }
        }
        if (isMatch) {
            List<RewardItem> rewardItemList = JSON.parseArray(collectDO.getRewards(), RewardItem.class);
            doReceiveActivityReward(rewardItemList, itemList, userId, collectDO, activityDO.getActivityId());
            return Result.ofSuccess();
        } else {
            return Result.ofOperationFailed("活动礼物数量不足");
        }
    }

    @Transactional(rollbackFor = Exception.class)
    public void doReceiveActivityReward(List<RewardItem> rewardItemList, List<GiftItem> itemList, Long userId, ActivityCollectDO collectDO, Long activityId) {
        String remarks = String.format("收集活动%s领取奖励", collectDO.getCollectName());
        Long orderId = identityService.getId();
        for (RewardItem rewardItem : rewardItemList) {
            rewardItem.getBagItems().forEach(i -> {
                Long expiredTime = -1L;
                if (i.getExpireTime() != null && i.getExpireTime() != -1L) {
                    expiredTime = System.currentTimeMillis() + i.getExpireTime();
                }
                // 赠送礼物
                bagService.addItem(userId, i.getItemId(), i.getItemType(), i.getQuantity(), expiredTime, remarks, orderId);
            });
        }
        BagItem bagItem = rewardItemList.get(0).getBagItems().get(0);
        List<UserBagRewardCostHisDO> hisDOList = new ArrayList<>(itemList.size());
        Long createTime = System.currentTimeMillis();
        for (GiftItem giftItem : itemList) {
            UserBagRewardCostHisDO tmp = new UserBagRewardCostHisDO();
            tmp.setHisId(identityService.getId());
            tmp.setItemId(giftItem.getGiftId());
            tmp.setUserId(userId);
            tmp.setQuantity(giftItem.getQuantity());
            tmp.setCollectId(collectDO.getCollectId());
            tmp.setTargetItemId(bagItem.getItemId());
            tmp.setCreateTime(createTime);
            tmp.setOrderId(orderId);
            tmp.setActivityId(activityId);
            hisDOList.add(tmp);
        }
        userBagRewardCostHisDAO.insertBatch(hisDOList);
    }


    /**
     * 收到礼物数量
     *
     * @param wallGiftIds
     * @param userId
     * @return
     */
    @Override
    public Map<Long, Integer> getWallGiftNumMap(List<Long> wallGiftIds, Long userId, ActivityDO activityDO, String collectId) {
        Map<Long, Integer> res = new HashMap<>();
        List<GiftWallItem> wallItems = userGiftWallService.getWallItems(userId,
                DateUtil.getIntDate(activityDO.getActivityBeginTime()).longValue(), DateUtil.getIntDate(activityDO.getActivityEndTime()).longValue(),
                wallGiftIds);
        List<UserBagRewardCostHisDO> userCost = userBagRewardCostHisDAO.getUserCostHis(userId, activityDO.getActivityId(), wallGiftIds);
        if (CollectionUtil.isNotEmpty(wallItems)) {
            Map<Long, Integer> userCostMap = new HashMap<>(userCost.size());
            if (CollectionUtil.isNotEmpty(userCost)) {
                userCost.forEach(i -> {
                    userCostMap.put(i.getItemId(), i.getQuantity());
                });
            }
            wallItems.forEach(i -> {
                Integer ownNum = res.get(i.getGiftId());
                if (ownNum == null) {
                    Integer sum = wallItems.stream().filter(e -> e.getGiftId().equals(i.getGiftId())).mapToInt(GiftWallItem::getQuantitySum).sum();
                    Integer tmp = userCostMap.get(i.getGiftId());
                    if (tmp == null) {
                        tmp = 0;
                    }
                    res.put(i.getGiftId(), sum - tmp);
                }
            });
        }
        return res;
    }

    /**
     * 获取用户背包ID
     *
     * @param giftIds
     * @param userId
     * @return
     */
    @Override
    public Map<Long, String> getGiftBagMap(List<Long> giftIds, Long userId) {
        List<UserBagDO> userBagDOList = bagService.getUserBagByGiftIds(giftIds, userId);
        Map<Long, String> res = new HashMap<>();
        if (CollectionUtil.isNotEmpty(userBagDOList)) {
            userBagDOList.forEach(i -> {
                res.put(i.getItemId(), i.getId());
            });
        }
        return res;
    }

    /**
     * 获取我自己的花费
     *
     * @param activityId
     * @param userId
     * @param activityBeginTime
     * @param activityEndTime
     * @return
     */
    @Override
    public ActivityStatRankDO getMySelfRank(Long activityId, Long userId, Long activityBeginTime, Long activityEndTime) {
        return activityLotteryHisDAO.getMySelfRank(activityId, userId, activityBeginTime, activityEndTime);
    }

    /**
     * 获取十一活动排行
     *
     * @param activityId
     * @param activityBeginTime
     * @param activityEndTime
     * @return
     */
    @Override
    public List<ActivityStatRankDO> getOctoberRank(Long activityId, Long activityBeginTime, Long activityEndTime) {
        return activityLotteryHisDAO.getOctoberRank(activityId, activityBeginTime, activityEndTime);
    }
}
