package com.starbuds.server.service.activity.impl;

import cn.hutool.core.convert.Convert;
import com.alibaba.fastjson.JSON;
import com.starbuds.server.common.constant.cache.RedisActivityKeys;
import com.starbuds.server.common.constant.finance.WalletTypeEnum;
import com.starbuds.server.common.constant.rocketmq.RocketTag;
import com.starbuds.server.common.constant.rocketmq.RocketTopic;
import com.starbuds.server.common.constant.statistics.StatPlatformTypeEnum;
import com.starbuds.server.common.pojo.activity.ActivityHeadlinePeriod;
import com.starbuds.server.common.pojo.activity.ActivityHeadlinePeriodVO;
import com.starbuds.server.common.pojo.activity.HeadlineLotteryRewardItem;
import com.starbuds.server.common.pojo.activity.HeadlineLotteryWinnerHisVO;
import com.starbuds.server.common.pojo.db.ActivityHeadlineConfigDO;
import com.starbuds.server.common.pojo.db.ActivityHeadlinePeriodItemDO;
import com.starbuds.server.common.pojo.mq.QueueHeadlineLotteryActivityParticipateMsg;
import com.starbuds.server.common.service.activity.HeadlineLotteryActivityService;
import com.starbuds.server.common.service.core.IdentityService;
import com.starbuds.server.common.service.data.RedisService;
import com.starbuds.server.common.service.finance.WalletService;
import com.starbuds.server.common.service.statistics.StatPlatformService;
import com.starbuds.server.service.activity.dao.ActivityHeadlineConfigDAO;
import com.starbuds.server.service.activity.dao.ActivityHeadlinePeriodDAO;
import com.starbuds.server.service.activity.dao.ActivityHeadlinePeriodItemDAO;
import com.starbuds.server.service.activity.headline.HeadlineLotteryHeatProvider;
import com.starbuds.server.service.activity.rocket.RocketProducer;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.apache.dubbo.config.annotation.Reference;
import org.apache.dubbo.config.annotation.Service;
import org.springframework.beans.BeanUtils;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.transaction.interceptor.TransactionAspectSupport;

import javax.annotation.Resource;
import java.math.BigDecimal;
import java.util.List;

/**
 * 头条抽奖服务实现
 *
 * @author Lay
 * @date 2020/12/28
 */
@Service(version = "1.0.0")
@Slf4j
public class HeadlineLotteryActivityServiceImpl implements HeadlineLotteryActivityService {

    @Reference(version = "1.0.0")
    IdentityService identityService;

    @Reference(version = "1.0.0")
    WalletService walletService;

    @Reference(version = "1.0.0")
    RedisService redisService;

    @Reference(version = "1.0.0")
    StatPlatformService statPlatformService;

    @Resource
    ActivityHeadlineConfigDAO activityHeadlineConfigDAO;

    @Resource
    ActivityHeadlinePeriodDAO activityHeadlinePeriodDAO;

    @Resource
    ActivityHeadlinePeriodItemDAO activityHeadlinePeriodItemDAO;

    @Resource
    HeadlineLotteryHeatProvider headlineLotteryHeatProvider;

    @Resource
    RocketProducer rocketProducer;

    /**
     * 判断期数是否存在
     *
     * @param configId
     * @param periodId
     * @return
     */
    @Override
    public boolean isPeriodExists(long configId, int periodId) {
        return activityHeadlinePeriodDAO.countById(configId, periodId) > 0;
    }

    /**
     * 判断活动是否进行中
     *
     * @param configId
     * @param periodId
     * @return
     */
    @Override
    public boolean isPeriodGoing(long configId, int periodId) {
        String goingPeriod = redisService.get(RedisActivityKeys.ACTIVITY_HEADLINE_LOTTERY_GOING);
        if (goingPeriod != null) {
            return goingPeriod.equals(String.format("%d:%d", configId, periodId));
        }
        return false;
    }

    /**
     * 获取活动实例
     *
     * @param configId
     * @param periodId
     * @return
     */
    @Override
    public ActivityHeadlinePeriodVO getPeriod(long configId, int periodId) {
        ActivityHeadlinePeriod period = activityHeadlinePeriodDAO.selectPeriodById(configId, periodId);
        if (period != null) {
            period.setHeat(headlineLotteryHeatProvider.get(configId, periodId));
            ActivityHeadlinePeriodVO periodVO = new ActivityHeadlinePeriodVO();
            BeanUtils.copyProperties(period, periodVO);
            periodVO.setRewardItems(JSON.parseArray(period.getPeriodRewardConfig(), HeadlineLotteryRewardItem.class));
            return periodVO;
        }
        return null;
    }

    /**
     * 获取进行中的活动实例
     *
     * @return
     */
    @Override
    public ActivityHeadlinePeriodVO getGoingPeriod() {
        String goingPeriod = redisService.get(RedisActivityKeys.ACTIVITY_HEADLINE_LOTTERY_GOING);
        if (StringUtils.isNotBlank(goingPeriod)) {
            String[] ids = goingPeriod.split(":");
            return getPeriod(Convert.toLong(ids[0]), Convert.toInt(ids[1]));
        }
        return null;
    }

    /**
     * 参加活动
     *
     * @param configId
     * @param periodId
     * @param userId
     * @param content
     * @param channelId
     * @return
     */
    @Transactional(rollbackFor = Exception.class)
    @Override
    public Long participate(long configId, int periodId, Long userId, String content, Long channelId) {
        // 查询活动
        ActivityHeadlinePeriod period = activityHeadlinePeriodDAO.selectPeriodById(configId, periodId);
        if (period == null) {
            return null;
        }

        try {
            Long balance = 0L;
            // 创建参加活动记录
            long itemId = identityService.getId();
            ActivityHeadlinePeriodItemDO item = new ActivityHeadlinePeriodItemDO();
            item.setItemId(itemId);
            item.setConfigId(configId);
            item.setPeriodId(periodId);
            item.setUserId(userId);
            item.setContent(content);
            item.setCreateTime(System.currentTimeMillis());
            activityHeadlinePeriodItemDAO.insert(item);

            // 扣钱
            if (period.getPrice() > 0) {
                balance = walletService.reduceCoin(userId, period.getPriceType(), period.getPrice(), String.format("%s-%s", period.getActivityName(), period.getConfigName()), itemId);
                if (balance == null || balance < 0) {
                    TransactionAspectSupport.currentTransactionStatus().setRollbackOnly();
                    return balance;
                }
            }

            // 更新热度值
            headlineLotteryHeatProvider.add(configId, periodId, 1L);

            // 发送消息
            try {
                QueueHeadlineLotteryActivityParticipateMsg participateMsg = new QueueHeadlineLotteryActivityParticipateMsg();
                participateMsg.setUserId(userId);
                participateMsg.setItemId(itemId);
                participateMsg.setContent(content);
                rocketProducer.sendMsg(RocketTopic.TPC_NORMAL, RocketTag.HEADLINE_LOTTERY_ACTIVITY_PARTICIPATE, participateMsg.serialize());
            } catch (Exception e) {
                log.error("头条抽奖活动参与消息发送失败", e);
            }

            // 统计
            if (WalletTypeEnum.STAR_DIAMOND.getValue() == period.getPriceType()) {
                statPlatformService.addPlatformStat(channelId, StatPlatformTypeEnum.HEADLINE_LOTTERY_ACTIVITY_PAY_AMOUNT.getValue(), BigDecimal.valueOf(period.getPrice()));
            }

            return balance;
        } catch (Exception e) {
            TransactionAspectSupport.currentTransactionStatus().setRollbackOnly();
            log.error("参加活动失败", e);
        }

        return null;
    }

    /**
     * 根据时间表达式查询活动配置
     *
     * @param timeExp
     * @return
     */
    @Override
    public ActivityHeadlineConfigDO getConfigByTimeExp(String timeExp) {
        return activityHeadlineConfigDAO.selectByTimeExp(timeExp);
    }

    /**
     * 根据 ID 获取配置
     *
     * @param configId
     * @return
     */
    @Override
    public ActivityHeadlineConfigDO getConfigById(long configId) {
        return activityHeadlineConfigDAO.selectById(configId);
    }

    /**
     * 查询往期获奖记录
     *
     * @param activityId
     * @return
     */
    @Override
    public List<HeadlineLotteryWinnerHisVO> getWinnerHisList(Long activityId) {
        return activityHeadlinePeriodDAO.selectWinnerHisList(activityId);
    }

    /**
     * 重置房间抢头条热度
     */
    @Override
    public void resetRoomHeat() {
        headlineLotteryHeatProvider.resetRoomHeat();
    }
}
