package com.starbuds.server.service.activity.impl;

import cn.hutool.core.lang.WeightRandom;
import cn.hutool.core.util.RandomUtil;
import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONArray;
import com.google.common.collect.ImmutableList;
import com.starbuds.server.common.constant.activity.LotteryGamePlayTypeEnum;
import com.starbuds.server.common.constant.activity.LotteryGameRewardTypeEnum;
import com.starbuds.server.common.constant.cache.RedisActivityKeys;
import com.starbuds.server.common.pojo.activity.bo.*;
import com.starbuds.server.common.pojo.data.PropsVO;
import com.starbuds.server.common.pojo.db.GiftDO;
import com.starbuds.server.common.pojo.db.LotteryGameSnapshotDO;
import com.starbuds.server.common.pojo.db.LotteryGameWinHisDO;
import com.starbuds.server.common.pojo.user.UserCardVO;
import com.starbuds.server.common.service.activity.LotteryGameService;
import com.starbuds.server.common.service.data.PropsService;
import com.starbuds.server.common.service.data.RedisService;
import com.starbuds.server.common.service.data.ReleaseConfigService;
import com.starbuds.server.common.service.media.GiftService;
import com.starbuds.server.common.service.user.UserService;
import com.starbuds.server.common.utils.LogUtil;
import com.starbuds.server.service.activity.dao.*;
import com.starbuds.server.service.activity.lotterygame.MarkUpEndServiceImpl;
import com.starbuds.server.service.activity.lotterygame.task.LotteryGameTimedTask;
import com.starbuds.server.service.activity.provider.LotteryGameProvider;
import com.starbuds.server.service.activity.provider.LotteryGameSnapshotProvider;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.collections.CollectionUtils;
import org.apache.commons.lang3.StringUtils;
import org.apache.commons.lang3.math.NumberUtils;
import org.apache.dubbo.config.annotation.Reference;
import org.apache.dubbo.config.annotation.Service;
import org.springframework.beans.BeanUtils;
import org.springframework.transaction.annotation.Transactional;

import javax.annotation.PostConstruct;
import javax.annotation.Resource;
import java.time.LocalDateTime;
import java.util.*;
import java.util.concurrent.TimeUnit;
import java.util.stream.Collectors;
import java.util.stream.IntStream;

import static com.google.common.base.Preconditions.checkState;
import static com.starbuds.server.service.activity.util.SpringContextUtil.isDev;
import static java.lang.Boolean.FALSE;
import static java.lang.Boolean.TRUE;

/**
 * @Author: lujiao
 * @Date: 2021/11/24 14:53
 */
@Service(version = "1.0.0")
@Slf4j
public class LotteryGameServiceImpl implements LotteryGameService {

    @Resource
    private LotteryGameTimerDAO timerDAO;

    @Reference(version = "1.0.0")
    private RedisService redisService;

    @Resource(name = "sendCountdownNoticeTask")
    private LotteryGameTimedTask sendCountdownNoticeTask;

    @Resource(name = "markUpActivityStartTask")
    private LotteryGameTimedTask markUpActivityStartTask;

    @Resource(name = "markUpActivityEndTask")
    private LotteryGameTimedTask markUpActivityEndTask;

    @Resource
    private LotteryGameProvider lotteryGameProvider;

    @Resource
    private LotteryGameSnapshotDAO lotteryGameSnapshotDAO;

    @Resource
    private MarkUpEndServiceImpl markUpEndService;

    @Resource
    private LotteryGameWinHisDAO lotteryGameWinHisDAO;

    @Reference(version = "1.0.0")
    private GiftService giftService;

    @Reference(version = "1.0.0")
    private PropsService propsService;

    @Reference(version = "1.0.0")
    private UserService userService;

    private List<LotteryGameTimedTask> timedTasks;

    @Resource
    private LotteryGameSnapshotProvider lotteryGameSnapshotProvider;

    @Resource
    private LotteryGameUserJoinHisDAO lotteryGameUserJoinHisDAO;

    @Resource
    private LotteryGameUserQualificationHisDAO lotteryGameUserQualificationHisDAO;

    @Reference(version = "1.0.0")
    private ReleaseConfigService releaseConfigService;

    @PostConstruct
    private void init() {
        timedTasks = ImmutableList.of(
                sendCountdownNoticeTask,
                markUpActivityStartTask,
                markUpActivityEndTask
        );
    }

    /**
     * 统计数据
     *
     * @param snapId
     * @param rewardNum
     * @param giveRewardTotal
     */
    @Override
    public void buildStatistic(Long snapId, Integer rewardNum, Long giveRewardTotal) {
        log.info("抽奖游戏统计 snapId : {} || rewardNum : {} || giveRewardTotal : {}", snapId, rewardNum, giveRewardTotal);
        Long timerId = lotteryGameSnapshotDAO.selectTimerIdBySnapId(snapId);

        // 参与人数
        Integer joinNum = lotteryGameUserJoinHisDAO.countById(snapId);
        // 活动结余

        Long totalRewardValue = 0L;
        List<LotteryGameRewardBO> rewards = lotteryGameSnapshotProvider.getRewards(snapId);
        if (CollectionUtils.isNotEmpty(rewards)) {
            for (LotteryGameRewardBO reward : rewards) {
                totalRewardValue = reward.getRewardTotalValue() + totalRewardValue;
            }
        }
        Long leftTotal = totalRewardValue - giveRewardTotal;
        log.info("抽奖游戏统计结果 timerId : {} || joinNum : {} || rewardNum : {} || giveRewardTotal : {} || leftTotal : {}", timerId, joinNum, rewardNum, giveRewardTotal, leftTotal);
        timerDAO.updateStatistic(timerId, joinNum, rewardNum, giveRewardTotal, leftTotal);
    }

    /**
     * 活动是否进行中
     *
     * @param snapId
     * @return true:进行中
     */
    @Override
    public boolean inProgressing(long snapId) {
        return StringUtils.isNotBlank(redisService.get(RedisActivityKeys.getLotteryGameStatus(snapId)));
    }

    /**
     * 是否被限流
     *
     * @param snapId
     * @return true 被限流
     */
    @Override
    public boolean isRateLimit(long snapId) {
        String rateLimitStr = releaseConfigService.getCachReleaseConfig("lotteryGameRateLimit");
        long rateLimit = NumberUtils.toLong(rateLimitStr, 100L);
        String totalRequestKey = RedisActivityKeys.getLotteryGameRateLimit(snapId, LocalDateTime.now().getSecond());
        boolean isNotExist = redisService.setIfAbsent(totalRequestKey, "1", 10, TimeUnit.SECONDS);
        if (isNotExist) {
            return false;
        }
        Long totalRequest = redisService.increase(totalRequestKey, 1L);
        if (isDev()) {
            log.info(LogUtil.devLog("限流 rateLimit={} < totalRequest={}"), rateLimit, totalRequest);
        }
        return rateLimit < totalRequest;
    }

    /**
     * 执行定时任务
     */
    @Override
    @Transactional(rollbackFor = Exception.class)
    public void doTimedTask() {
        timedTasks.parallelStream().forEach(LotteryGameTimedTask::doTask);
    }

    /**
     * 获得奖项总数量
     *
     * @param snapId
     * @return
     */
    @Override
    public int getRewardCount(long snapId) {
        return lotteryGameProvider.getRewardCount(snapId);
    }

    /**
     * 抽奖游戏进行中的快照集合
     *
     * @return
     */
    @Override
    public List<LotteryGameSnapshotDO> getInProgressingSnaps() {
        return Optional.ofNullable(lotteryGameProvider.getInProgressingSnaps()).orElse(Collections.emptyList());
    }

    /**
     * 标记抽奖游戏结束
     *
     * @param snapId
     * @return true:成功
     */
    @Override
    public boolean markUpEnd(long snapId) {
        List<LotteryGameSnapshotDO> snaps = lotteryGameProvider.getInProgressingSnaps();
        if (CollectionUtils.isEmpty(snaps)) {
            log.warn("标记抽奖游戏结束失败，当前没有正在进行中的活动 snapId={}", snapId);
            return false;
        }
        Optional<LotteryGameSnapshotDO> op = snaps.stream().filter(snap -> snap.getSnapId() - snapId == 0).findAny();
        if (!op.isPresent()) {
            log.warn("标记抽奖游戏结束失败，当前活动没有在正在进行中的活动列表中 snapId={}", snapId);
            return false;
        }
        LotteryGameSnapshotDO snap = op.get();
        if (!markUpEndService.canEnd(snapId, System.currentTimeMillis(), snap.getMinEndTime(), snap.getMaxEndTime())) {
            log.warn("标记抽奖游戏结束失败，当前活动不满足结束条件 snapId={}", snapId);
            return false;
        }

        return markUpEndService.markUpEnd(snapId);
    }

    @Deprecated
    public LotteryGameWinItemBO lotteryOld(long snapId) {
        String key = RedisActivityKeys.getLotteryGameWinItems(snapId);
        String jsonStr = redisService.lLeftPop(key);
        if (StringUtils.isBlank(jsonStr)) {
            log.error("抽奖失败 jsonStr is blank");
            return null;
        }
        LotteryGameWinItemBO item = JSON.parseObject(jsonStr, LotteryGameWinItemBO.class);
        if (item == null) {
            log.error("抽奖失败 item is null");
            return null;
        }
        if (LotteryGameRewardTypeEnum.toEnum(item.getRewardType()) == LotteryGameRewardTypeEnum.NONE) {
            return null;
        }
        Long rewardPrice = Optional.ofNullable(item.getRewardPrice()).orElse(0L);
        item.setRewardPrice(rewardPrice);
        return item;
    }

    @Override
    public LotteryGameWinItemBO lottery(long snapId) {
        Map<String, LotteryGameWinItemBO> winItemMap = getWinItemMap(snapId);
        List<String> rewardKeys = getRewardKeys(snapId, winItemMap);
        if (CollectionUtils.isEmpty(rewardKeys)) {
            return null;
        }

        Collections.shuffle(rewardKeys);
        String rewardKey;
        int rewardType = 0;
        long rewardId = 0;
        Long stock = -1L;
        for (int i = 0; i < rewardKeys.size(); i++) {
            rewardKey = rewardKeys.get(i);

            String replace = StringUtils.replace(rewardKey, String.format("lottery:game:reward:stock:%d:", snapId), "");
            String[] split = StringUtils.split(replace, ":");
            rewardType = NumberUtils.toInt(split[0]);
            rewardId = NumberUtils.toLong(split[1]);
            String field = RedisActivityKeys.getLotteryGameWinItemsMapField(rewardType, rewardId);

            Long rewardNum = Long.parseLong(Math.abs(winItemMap.get(field).getRewardNum()) + "");
            stock = redisService.increase(rewardKey, -rewardNum);
            if (stock > -1) {
                break;
            }
        }
        if (stock < 0) {
            return null;
        }

        return winItemMap.get(RedisActivityKeys.getLotteryGameWinItemsMapField(rewardType, rewardId));

        /*String key = RedisActivityKeys.getLotteryGameRewards(snapId);
        String rewardKey = cacheService.lLeftPop(key);
        if (StringUtils.isBlank(rewardKey)) {
            return null;
        }

        Map<String, LotteryGameWinItemBO> winItemMap = getWinItemMap(snapId);
        return winItemMap.get(rewardKey);*/
    }

    @Transactional(rollbackFor = Exception.class)
    @Override
    public LotteryGameWinHisDO lotteryAndSaveWinHis(long snapId, LotteryGamePlayTypeEnum playType, long userId) {
        LotteryGameWinItemBO winItem = lottery(snapId);
        if (winItem == null) {
            return null;
        }
        LotteryGameRewardTypeEnum rewardTypeEnum = LotteryGameRewardTypeEnum.toEnum(winItem.getRewardType());
        if (LotteryGameRewardTypeEnum.NONE == rewardTypeEnum) {
            return null;
        }

        LotteryGameWinHisDO winHis = getWinHis(snapId, playType, userId, winItem);
        int insertWinHis = lotteryGameWinHisDAO.insert(winHis);
        checkState(insertWinHis == 1);

        String winItemsKey = RedisActivityKeys.getLotteryGameWinHisItems(snapId, userId);
        JSONArray winItems = getWinItems(winHis, winItemsKey);
        winItems = mergeWinItems(winItems);
        redisService.set(winItemsKey, JSON.toJSONString(winItems), 6, TimeUnit.HOURS);

        return winHis;
    }

    @Override
    public void lotteryAndSaveWinHis(long snapId, LotteryGamePlayTypeEnum playType, long userId, int count) {
        IntStream.range(0, count)
                .forEach(i -> lotteryAndSaveWinHis(snapId, playType, userId));
    }

    private JSONArray getWinItems(LotteryGameWinHisDO winHis, String winHisArrayKey) {
        // [奖品id-物品类型-数量]
        String item = String.format("%s-%s-%s", winHis.getRewardId(), winHis.getRewardType(), winHis.getRewardNum());
        JSONArray winHisArray;
        String winHisArrayStr = redisService.get(winHisArrayKey);
        if (StringUtils.isBlank(winHisArrayStr)) {
            winHisArray = new JSONArray();
        } else {
            winHisArray = JSON.parseArray(winHisArrayStr);
        }
        winHisArray.add(item);
        return winHisArray;
    }

    private JSONArray mergeWinItems(JSONArray winItems) {
        Map<String, Integer> itemCountMap = new HashMap<>();
        for (int i = 0; i < winItems.size(); i++) {
            String item = winItems.getString(i);
            String[] items = StringUtils.split(item, "-");
            String key = String.format("%s-%s", items[0], items[1]);
            int addCount = NumberUtils.toInt(items[2]);
            Integer count = itemCountMap.get(key);
            if (count == null) {
                count = 0;
            }
            count += addCount;
            itemCountMap.put(key, count);
        }

        JSONArray array = new JSONArray();
        itemCountMap.forEach((k, v) -> array.add(k + "-" + v));
        return array;
    }

    private LotteryGameWinHisDO getWinHis(long snapId, LotteryGamePlayTypeEnum playType, long userId,
                                          LotteryGameWinItemBO winItem) {
        LotteryGameWinHisDO winHis = new LotteryGameWinHisDO();
        winHis.setRewardType(winItem.getRewardType());
        winHis.setRewardId(winItem.getRewardId());
        winHis.setRewardNum(winItem.getRewardNum());
        winHis.setRewardPrice(winItem.getRewardPrice());
        winHis.setRewardTotalPrice(winHis.getRewardNum() * winHis.getRewardPrice());
        winHis.setSnapId(snapId);
        winHis.setPlayType(playType.value);
        winHis.setUserId(userId);
        winHis.setCreateTime(System.currentTimeMillis());
        winHis.setRewardName("-");
        return winHis;
    }


    private Map<String, LotteryGameWinItemBO> getWinItemMap(long snapId) {
        String key = RedisActivityKeys.getLotteryGameWinItemsMap(snapId);
        Map<Object, Object> map = redisService.hashGetAll(key);
        Map<String, LotteryGameWinItemBO> winItemMap = new HashMap<>();
        map.forEach((k, v) -> {
            LotteryGameWinItemBO winItem = JSON.parseObject((String) v, LotteryGameWinItemBO.class);
            winItemMap.put((String) k, winItem);
        });
        return winItemMap;
    }

    /**
     * 获得库存大于0的奖项id集合
     *
     * @param snapId
     * @param winItemMap
     * @return
     */
    private List<String> getRewardKeys(long snapId, final Map<String, LotteryGameWinItemBO> winItemMap) {
        List<String> rewardKeys = winItemMap.values()
                .stream()
                .map(v -> RedisActivityKeys.getLotteryGameRewardStock(snapId, v.getRewardType(), v.getRewardId()))
                .collect(Collectors.toList());
        return rewardKeys.stream()
                .filter(key -> {
                    int count = NumberUtils.toInt(redisService.get(key));
                    return count > 0;
                })
                .collect(Collectors.toList());
    }

    /**
     * 查询抽奖游戏开奖结果-红包
     *
     * @param userId
     * @param snapId
     * @return
     */
    @Override
    public LotteryGameResultRedBag getResultByRedBag(long userId, long snapId) {
        List<LotteryGameWinHisDO> top10List = lotteryGameWinHisDAO.selectListBySnapId(snapId, 10);
        if (CollectionUtils.isEmpty(top10List)) {
            return LotteryGameResultRedBag.builder().build().empty();
        }

        Optional<LotteryGameWinHisDO> myWin = ownWinHis(userId, snapId);
        List<LotteryGameWinHisDO> tops = mergeTops(myWin, top10List);

        return LotteryGameResultRedBag.builder()
                .winInfos(redBagWinInfos(tops))
                .winCount(lotteryGameWinHisDAO.selectWinUserCountBySnapId(snapId))
                .build();
    }

    /**
     * 查询抽奖游戏开奖结果-瓜分蛋糕
     *
     * @param userId
     * @param snapId
     * @return
     */
    @Override
    public LotteryGameResultDivideCake getResultByDivideCake(long userId, long snapId) {
        List<LotteryGameWinHisDO> userWinList = lotteryGameWinHisDAO.selectListBySnapIdUserId(snapId, userId, Integer.MAX_VALUE);
        if (CollectionUtils.isEmpty(userWinList)) {
            return LotteryGameResultDivideCake.builder().build().empty();
        }
        List<RedBagWinInfo> redBagWinInfos = redBagWinInfos(userWinList);
        List<DivideCakeWinInfo> divideCakeWinInfos = getDivideCakeWinInfos(redBagWinInfos);
        divideCakeWinInfos = mergeDivideCakeWinInfos(divideCakeWinInfos);
        divideCakeWinInfos = divideCakeWinInfos.stream()
                .sorted(Comparator
                        .comparingLong(DivideCakeWinInfo::getTotalPrice)
                        .reversed())
                .collect(Collectors.toList());

        LotteryGameResultDivideCake result = LotteryGameResultDivideCake.builder()
                .winInfos(divideCakeWinInfos)
                .build();
        if (isDev()) {
            log.info("抽奖游戏开奖结果瓜分蛋糕结果 {}", LogUtil.obj2JsonObj(result));
        }
        return result;
    }

    /**
     * 获得自己的中奖记录
     *
     * @param userId
     * @param snapId
     * @return
     */
    private Optional<LotteryGameWinHisDO> ownWinHis(long userId, long snapId) {
        List<LotteryGameWinHisDO> userWinList = lotteryGameWinHisDAO.selectListBySnapIdUserId(snapId, userId, 1);
        if (CollectionUtils.isEmpty(userWinList)) {
            return Optional.empty();
        }
        return Optional.ofNullable(userWinList.get(0));
    }

    /**
     * 合并展示排名列表
     *
     * @param myWinHis
     * @param top10List
     * @return
     */
    private List<LotteryGameWinHisDO> mergeTops(Optional<LotteryGameWinHisDO> myWinHis, List<LotteryGameWinHisDO> top10List) {
        if (CollectionUtils.isEmpty(top10List)) {
            top10List = Collections.emptyList();
        }
        if (!myWinHis.isPresent()) {
            return top10List;
        }

        LotteryGameWinHisDO myWin = myWinHis.get();
        List<LotteryGameWinHisDO> exclusionMyselfList = top10List.stream()
                .filter(his -> his.getUserId() - myWin.getUserId() != 0)
                .collect(Collectors.toList());

        List<LotteryGameWinHisDO> result = new ArrayList<>();
        result.add(myWin);
        result.addAll(exclusionMyselfList);
        return result;
    }

    private List<RedBagWinInfo> redBagWinInfos(List<LotteryGameWinHisDO> tops) {
        List<Long> userIds = tops.stream().map(his -> his.getUserId()).distinct().collect(Collectors.toList());
        List<Long> giftIds = getItemIds(tops, LotteryGameRewardTypeEnum.GIFT);
        List<Long> propsIds = getItemIds(tops, LotteryGameRewardTypeEnum.PROPS);

        Map<Long, PropsVO> propsMap = propsService.getPropsVOMap(propsIds);
        Map<Long, GiftDO> giftMap = giftService.getGiftMapByGiftIds(giftIds);
        Map<Long, UserCardVO> userMap = userService.getCardMap(userIds.stream().collect(Collectors.toSet()));

        return tops.stream()
                .map(his -> his2RedBagVO(his.getUserId(), his.getRewardId(),
                        LotteryGameRewardTypeEnum.toEnum(his.getRewardType()),
                        his.getRewardNum(),
                        userMap, giftMap, propsMap, his.getRewardTotalPrice()))
                .collect(Collectors.toList());
    }

    /**
     * 获得奖品id
     *
     * @param winItems
     * @param typeEnum
     * @return
     */
    private List<Long> getItemIds(List<LotteryGameWinHisDO> winItems, LotteryGameRewardTypeEnum typeEnum) {
        return winItems.stream()
                .filter(Objects::nonNull)
                .filter(item -> LotteryGameRewardTypeEnum.toEnum(item.getRewardType()) == typeEnum)
                .map(item -> item.getRewardId())
                .distinct()
                .collect(Collectors.toList());
    }

    private RedBagWinInfo his2RedBagVO(Long userId, Long rewardId, LotteryGameRewardTypeEnum type,
                                       Integer quantity,
                                       Map<Long, UserCardVO> userMap,
                                       Map<Long, GiftDO> giftMap,
                                       Map<Long, PropsVO> propsMap,
                                       Long totalPrice) {
        String userName = "";
        String userAvatar = "";
        UserCardVO user = userMap.get(userId);
        if (user != null) {
            userName = user.getUserName();
            userAvatar = user.getUserAvatar();
        }
        String prizeName = "";
        String prizeIcon = "";
        switch (type) {
            case GIFT:
                GiftDO gift = giftMap.get(rewardId);
                if (gift != null) {
                    prizeName = gift.getGiftName();
                    prizeIcon = gift.getGiftIcon();
                }
                break;
            case PROPS:
                PropsVO props = propsMap.get(rewardId);
                if (props != null) {
                    prizeName = props.getPropsName();
                    prizeIcon = props.getPropsIcon();
                }
                break;
        }

        RedBagWinInfo vo = new RedBagWinInfo();
        vo.setUserId(userId);
        vo.setUserName(Optional.ofNullable(userName).orElse(""));
        vo.setUserAvatar(Optional.ofNullable(userAvatar).orElse(""));
        vo.setPrizeName(Optional.ofNullable(prizeName).orElse(""));
        vo.setPrizeIcon(Optional.ofNullable(prizeIcon).orElse(""));
        vo.setQuantity(Optional.ofNullable(quantity).orElse(0));
        vo.setTotalPrice(totalPrice);
        return vo;
    }


    /**
     * 获取等待中的队列
     *
     * @return
     */
    @Override
    public List<LotteryGameSnapshotDO> getWaitLotteryGameList() {
        Long now = System.currentTimeMillis();
        return lotteryGameSnapshotDAO.getWaitLotteryGameList(now);
    }

    /**
     * 数据是否存在
     *
     * @param snapId
     * @param userId
     * @return true 存在
     */
    @Override
    public boolean existBySnapIdUserId(Long snapId, Long userId) {
        Integer exist = lotteryGameUserQualificationHisDAO.selectExistBySnapIdUserId(snapId, userId);
        return exist != null;
    }

    /**
     * 是否有效点击
     *
     * @param clickRate 点中概率
     * @return true 有效
     */
    @Override
    public boolean effectiveClick(double clickRate) {
        List<WeightRandom.WeightObj<Boolean>> weightList = ImmutableList.of(
                new WeightRandom.WeightObj<>(TRUE, clickRate),
                new WeightRandom.WeightObj<>(FALSE, 1 - clickRate)
        );
        return RandomUtil.weightRandom(weightList).next() == TRUE;
    }

    private List<DivideCakeWinInfo> mergeDivideCakeWinInfos(List<DivideCakeWinInfo> divideCakeWinInfos) {
        Map<String, DivideCakeWinInfo> map = new HashMap<>();
        divideCakeWinInfos.stream()
                .forEach(win -> {
                    String prizeName = win.getPrizeName();
                    DivideCakeWinInfo info = map.get(prizeName);
                    if (info == null) {
                        info = win;
                        map.put(prizeName, info);
                        return;
                    }

                    Integer quantity = info.getQuantity() + win.getQuantity();
                    Long totalPrice = info.getTotalPrice() + win.getTotalPrice();
                    info.setQuantity(quantity);
                    info.setTotalPrice(totalPrice);
                    map.put(prizeName, info);
                });

        return map.values()
                .stream()
                .collect(Collectors.toList());
    }

    private List<DivideCakeWinInfo> getDivideCakeWinInfos(List<RedBagWinInfo> redBagWinInfos) {
        return redBagWinInfos.stream()
                .map(redBag -> {
                    DivideCakeWinInfo divideCakeWinInfo = new DivideCakeWinInfo();
                    BeanUtils.copyProperties(redBag, divideCakeWinInfo);
                    return divideCakeWinInfo;
                })
                .collect(Collectors.toList());
    }
}
