package com.starbuds.server.service.activity.impl;

import com.alibaba.fastjson.JSON;
import com.starbuds.server.common.constant.activity.LotteryGamePlayTypeEnum;
import com.starbuds.server.common.constant.activity.LotteryGameRewardTypeEnum;
import com.starbuds.server.common.constant.cache.RedisActivityKeys;
import com.starbuds.server.common.pojo.activity.bo.LotteryGameRewardBO;
import com.starbuds.server.common.pojo.activity.qo.LotteryGameSnapshotPageListQO;
import com.starbuds.server.common.pojo.activity.vo.LotteryGameRewardVO;
import com.starbuds.server.common.pojo.activity.vo.LotteryGameSummary;
import com.starbuds.server.common.pojo.api.PageList;
import com.starbuds.server.common.pojo.data.PropsVO;
import com.starbuds.server.common.pojo.db.LotteryGameSnapshotDO;
import com.starbuds.server.common.pojo.media.GiftItemVO;
import com.starbuds.server.common.service.activity.LotteryGameSnapshotService;
import com.starbuds.server.common.service.data.PropsService;
import com.starbuds.server.common.service.data.RedisService;
import com.starbuds.server.common.service.finance.WalletService;
import com.starbuds.server.common.service.media.GiftService;
import com.starbuds.server.common.utils.LogUtil;
import com.starbuds.server.service.activity.dao.LotteryGameDAO;
import com.starbuds.server.service.activity.dao.LotteryGameSnapshotDAO;
import com.starbuds.server.service.activity.provider.LotteryGameConfigItemProvider;
import com.starbuds.server.service.activity.provider.LotteryGameConfigProvider;
import com.starbuds.server.service.activity.provider.LotteryGameSnapshotProvider;
import com.starbuds.server.service.activity.provider.LotteryGameTimerProvider;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.collections.CollectionUtils;
import org.apache.commons.lang3.StringUtils;
import org.apache.dubbo.config.annotation.Reference;
import org.apache.dubbo.config.annotation.Service;
import org.springframework.beans.BeanUtils;

import javax.annotation.Resource;
import java.util.*;
import java.util.concurrent.TimeUnit;
import java.util.stream.Collectors;

import static com.starbuds.server.service.activity.util.SpringContextUtil.isDev;

/**
 * 抽奖游戏活动执行快照
 */
@Service(version = "1.0.0")
@Slf4j
public class LotteryGameSnapshotServiceImpl implements LotteryGameSnapshotService {

    @Resource
    private LotteryGameSnapshotProvider lotteryGameSnapshotProvider;

    @Resource
    private LotteryGameTimerProvider lotteryGameTimerProvider;

    @Resource
    private LotteryGameConfigItemProvider lotteryGameConfigItemProvider;

    @Resource
    private LotteryGameConfigProvider lotteryGameConfigProvider;

    @Resource
    private LotteryGameDAO lotteryGameDAO;

    @Resource
    private LotteryGameSnapshotDAO lotteryGameSnapshotDAO;

    @Reference(version = "1.0.0")
    private GiftService giftService;

    @Reference(version = "1.0.0")
    private PropsService propsService;

    @Reference(version = "1.0.0")
    private WalletService walletService;

    @Reference(version = "1.0.0")
    private RedisService redisService;

    @Override
    public LotteryGameSnapshotDO getById(Long id) {
        return lotteryGameSnapshotProvider.getById(id);
    }

    @Override
    public boolean exist(Long id) {
        return lotteryGameSnapshotProvider.exist(id);
    }

    @Override
    public PageList<LotteryGameSnapshotDO> getPageList(LotteryGameSnapshotPageListQO qo) {
        return lotteryGameSnapshotProvider.getPageList(qo);
    }

    @Override
    public Map<Long, LotteryGameSnapshotDO> getMapByIds(Collection ids) {
        return lotteryGameSnapshotProvider.getMapByIds(ids);
    }

    /**
     * 抽奖游戏活动摘要信息
     *
     * @param snapId
     * @return
     */
    @Override
    public LotteryGameSummary getSummary(long snapId) {
        LotteryGameSnapshotDO po = lotteryGameSnapshotDAO.selectLotteryGameSummary(snapId);
        if (po == null) {
            return null;
        }
        String rewardPool = po.getRewardPool();
        if (StringUtils.isBlank(rewardPool)) {
            log.error("rewardPool is blank ! po=" + po);
            return null;
        }

        LotteryGameSummary summary = new LotteryGameSummary();
        BeanUtils.copyProperties(po, summary);
        summary.setSnapId(snapId);
        List<LotteryGameRewardBO> rewards = JSON.parseArray(rewardPool, LotteryGameRewardBO.class);
        summary.setRewards(getRewardVOList(rewards));
        summary.setPlayType(LotteryGamePlayTypeEnum.toEnum(po.getPlayType()).value);
        return summary;
    }

    @Override
    public LotteryGameSummary getSummaryByCache(long snapId) {
        String key = RedisActivityKeys.getLotteryGameSummary(snapId);
        String value = redisService.get(key);
        if (StringUtils.isNotBlank(value)) {
            return JSON.parseObject(value, LotteryGameSummary.class);
        }

        LotteryGameSummary summary = getSummary(snapId);
        if (summary == null) {
            log.error("抽奖游戏活动摘要信息(从缓存中获取)失败，无效的snapId=" + snapId);
            return null;
        }

        redisService.set(key, JSON.toJSONString(summary), 10, TimeUnit.MINUTES);
        return summary;
    }

    private List<LotteryGameRewardVO> getRewardVOList(List<LotteryGameRewardBO> rewards) {
        Map<LotteryGameRewardTypeEnum, List<LotteryGameRewardBO>> typeMap = new HashMap<>();
        for (LotteryGameRewardBO reward : rewards) {
            LotteryGameRewardTypeEnum type = LotteryGameRewardTypeEnum.toEnum(reward.getRewardType());
            if (type == null) {
                continue;
            }
            List<LotteryGameRewardBO> list = typeMap.get(type);
            if (list == null) {
                list = new ArrayList<>();
                typeMap.put(type, list);
            }
            list.add(reward);
        }

        List<LotteryGameRewardBO> gifts = typeMap.get(LotteryGameRewardTypeEnum.GIFT);
        List<LotteryGameRewardBO> props = typeMap.get(LotteryGameRewardTypeEnum.PROPS);

        List<LotteryGameRewardVO> result = new ArrayList<>();
        result.addAll(getRewardByGifts(gifts));
        result.addAll(getRewardByProps(props));
        result = result.stream()
                .sorted(Comparator.comparing(LotteryGameRewardVO::getRewardValue).reversed())
                .collect(Collectors.toList());
        return result;
    }

    private List<LotteryGameRewardVO> getRewardByGifts(List<LotteryGameRewardBO> gifts) {
        List<Long> giftIds = getRewardIds(gifts);
        List<GiftItemVO> list = giftService.getGiftItemsByIds(giftIds);
        if (CollectionUtils.isEmpty(list)) {
            return Collections.emptyList();
        }
        return list.stream()
                .map(gift -> {
                    LotteryGameRewardVO vo = new LotteryGameRewardVO();
                    vo.setRewardIcon(gift.getGiftIcon());
                    vo.setRewardName(gift.getGiftName());
                    vo.setRewardValue(Optional.ofNullable(gift.getGiftPrice()).orElse(0L));
                    vo.setRewardValueType(gift.getGiftPriceTypeName());
                    return vo;
                })
                .collect(Collectors.toList());
    }

    private List<LotteryGameRewardVO> getRewardByProps(List<LotteryGameRewardBO> props) {
        List<Long> propIds = getRewardIds(props);
        List<PropsVO> list = propsService.getPropsByIds(propIds);
        if (CollectionUtils.isEmpty(list)) {
            return Collections.emptyList();
        }
        Map<Integer, String> walletTypeNameMap = walletService.getWalletTypeNameMap();
        if (isDev()) {
            log.info("walletTypeNameMap = {} props list {}", LogUtil.obj2JsonObj(walletTypeNameMap), LogUtil.obj2JsonObj(list));
        }
        return list.stream()
                .filter(Objects::nonNull)
                .map(prop -> {
                    LotteryGameRewardVO vo = new LotteryGameRewardVO();
                    vo.setRewardIcon(prop.getPropsIcon());
                    vo.setRewardName(prop.getPropsName());
                    vo.setRewardValue(Optional.ofNullable(prop.getPropsExPrice()).orElse(0L));
                    vo.setRewardValueType(Optional.ofNullable(walletTypeNameMap.get(prop.getPropsExPriceType())).orElse(""));
                    return vo;
                })
                .collect(Collectors.toList());
    }

    private List<Long> getRewardIds(List<LotteryGameRewardBO> rewards) {
        if (CollectionUtils.isEmpty(rewards)) {
            return Collections.emptyList();
        }
        return rewards.stream().filter(Objects::nonNull)
                .filter(reward -> reward.getRewardId() != null)
                .map(LotteryGameRewardBO::getRewardId)
                .distinct()
                .collect(Collectors.toList());
    }
}