package com.starbuds.server.service.activity.lotterygame.task;

import com.alibaba.fastjson.JSON;
import com.starbuds.server.common.constant.activity.LotteryGamePlayTypeEnum;
import com.starbuds.server.common.constant.activity.LotteryGameRewardTypeEnum;
import com.starbuds.server.common.constant.cache.RedisActivityKeys;
import com.starbuds.server.common.constant.rocketmq.RocketTag;
import com.starbuds.server.common.constant.rocketmq.RocketTopic;
import com.starbuds.server.common.pojo.activity.bo.LotteryGameRewardBO;
import com.starbuds.server.common.pojo.activity.bo.LotteryGameRewardSummary;
import com.starbuds.server.common.pojo.activity.bo.LotteryGameWinItemBO;
import com.starbuds.server.common.pojo.data.PropsVO;
import com.starbuds.server.common.pojo.db.GiftDO;
import com.starbuds.server.common.pojo.db.LotteryGameSnapshotDO;
import com.starbuds.server.common.pojo.media.bo.PriceInfoBO;
import com.starbuds.server.common.pojo.message.im.activity.LotteryGameCountdownImMsg;
import com.starbuds.server.common.pojo.mq.statcommonbiz.QueueDivideCakeSetupGiftTotalValueMsg;
import com.starbuds.server.common.service.activity.LotteryGameSnapshotService;
import com.starbuds.server.common.service.data.PropsService;
import com.starbuds.server.common.service.data.RedisService;
import com.starbuds.server.common.service.media.GiftService;
import com.starbuds.server.common.service.message.biz.LotteryGameImMsgService;
import com.starbuds.server.common.utils.LogUtil;
import com.starbuds.server.service.activity.dao.LotteryGameDAO;
import com.starbuds.server.service.activity.dao.LotteryGameSnapshotDAO;
import com.starbuds.server.service.activity.provider.LotteryGameSnapshotProvider;
import com.starbuds.server.service.activity.rocket.RocketProducer;
import com.starbuds.server.service.activity.util.TimeUtil;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.collections.CollectionUtils;
import org.apache.commons.lang3.StringUtils;
import org.apache.dubbo.config.annotation.Reference;
import org.springframework.beans.BeanUtils;
import org.springframework.stereotype.Component;
import org.springframework.transaction.annotation.Transactional;

import javax.annotation.Resource;
import java.util.Collections;
import java.util.List;
import java.util.Map;
import java.util.Objects;
import java.util.concurrent.TimeUnit;
import java.util.stream.Collectors;
import java.util.stream.IntStream;

import static com.google.common.base.Preconditions.checkArgument;
import static com.google.common.base.Preconditions.checkNotNull;
import static com.starbuds.server.common.constant.activity.LotteryGameTimerStatusEnum.IN_PROGRESS;
import static com.starbuds.server.common.constant.activity.LotteryGameTimerStatusEnum.STANDBY;

/**
 * 发送倒计时通知
 *
 * @author aoe
 * @date 2021/11/25
 */
@Component
@Slf4j
public class SendCountdownNoticeTask implements LotteryGameTimedTask {

    @Resource
    private LotteryGameSnapshotDAO lotteryGameSnapshotDAO;

    @Reference(version = "1.0.0")
    private LotteryGameImMsgService lotteryGameImMsgService;

    @Resource
    private LotteryGameDAO lotteryGameDAO;

    @Resource
    private LotteryGameSnapshotProvider lotteryGameSnapshotProvider;

    @Reference(version = "1.0.0")
    private RedisService redisService;

    @Reference(version = "1.0.0")
    private GiftService giftService;

    @Reference(version = "1.0.0")
    private PropsService propsService;

    @Resource
    private RocketProducer rocketProducer;

    @Resource
    private LotteryGameSnapshotService lotteryGameSnapshotService;

    @Override
    @Transactional(rollbackFor = Exception.class)
    public boolean doTask() {
        final long now = System.currentTimeMillis();
        String currentStr = TimeUtil.currentStr(now);
        Long timerId = lotteryGameDAO.selectSendCountdownMsgTimerId(currentStr, STANDBY.value);
        if (timerId == null) {
            return false;
        }

        LotteryGameSnapshotDO snap = createLotteryGameSnapshot(timerId);
        if (snap == null) {
            return false;
        }
        Long snapId = snap.getSnapId();
        // 刷新数据到缓存，防止客户端请求过快数据库还没同步数据
        lotteryGameSnapshotService.getSummaryByCache(snapId);

        sendCountdownNotice(snap);

        final List<LotteryGameRewardBO> rewards = lotteryGameSnapshotProvider.getRewards(snapId);
        if (CollectionUtils.isEmpty(rewards)) {
            log.error("保存奖项总数量到缓存失败，rewards is empty snapId=" + snapId);
            return false;
        }
        // 保存奖项总数量到缓存
        saveRewardCountToCache(snapId, rewards);

        List<Long> giftIds = getRewardIds(rewards, LotteryGameRewardTypeEnum.GIFT);
        List<Long> propsIds = getRewardIds(rewards, LotteryGameRewardTypeEnum.PROPS);
        Map<Long, PropsVO> propsMap = propsService.getPropsVOMap(propsIds);
        Map<Long, GiftDO> giftMap = giftService.getGiftMapByGiftIds(giftIds);
        // 保存抽奖游戏奖品信息摘要到缓存
        saveRewardSummariesToCache(snapId, rewards, propsMap, giftMap);
        saveWinItemsInRedisMap(snapId, rewards, propsMap, giftMap);
        // 保存奖品库存到缓存
        saveRewardsStockToCache(snapId, rewards);

        LotteryGamePlayTypeEnum playType = LotteryGamePlayTypeEnum.toEnum(snap.getPlayType());
        if (LotteryGamePlayTypeEnum.DIVIDE_CAKE == playType) {
            sendSetupGiftTotalValueMsg(snapId, rewards);
        }
        // 保存所有奖品到缓存
//        saveAllRewardsToCache(snapId, rewards);

        return true;
    }

    /**
     * 创建抽奖游戏快照
     *
     * @param timerId
     * @return
     */
    @Transactional(rollbackFor = Exception.class)
    public LotteryGameSnapshotDO createLotteryGameSnapshot(Long timerId) {
        log.info(LogUtil.devLog("准备创建抽奖游戏快照 timerId={}"), timerId);

        LotteryGameSnapshotDO snap = lotteryGameSnapshotProvider.createLotteryGameSnapshot(timerId);
        checkNotNull(snap, "创建抽奖游戏活动执行快照失败 timerId=" + timerId);

        int updateCount = updateTimer2InProgress(timerId);
        checkArgument(updateCount == 1, "更新抽奖定时器配置状态为进行中失败 timerId=" + timerId);
        return snap;
    }

    /**
     * 更新抽奖定时器配置状态为进行中
     *
     * @param timerId
     * @return
     */
    @Transactional(rollbackFor = Exception.class)
    public int updateTimer2InProgress(long timerId) {
        return lotteryGameDAO.updateTimer2InProgress(timerId, IN_PROGRESS.value, STANDBY.value);
    }

    /**
     * 发送倒计时通知
     *
     * @param snap
     * @return
     */
    private boolean sendCountdownNotice(LotteryGameSnapshotDO snap) {
        long countdown = snap.getCountDown() / 60;
        String msg = StringUtils.replace(snap.getNotice(), "#分钟#", countdown + "");

        LotteryGameCountdownImMsg imMsg = new LotteryGameCountdownImMsg();
        imMsg.setSnapId(snap.getSnapId());
        imMsg.setStartTime(snap.getStartTime());
        imMsg.setMinEndTime(snap.getMinEndTime());
        imMsg.setMaxEndTime(snap.getMaxEndTime());
        imMsg.setMsg(msg);
        imMsg.setPlayType(LotteryGamePlayTypeEnum.toEnum(snap.getPlayType()).value);
        boolean send = lotteryGameImMsgService.sendCountdownImMsg(imMsg);

        String imMsgStr = JSON.toJSONString(imMsg);
        if (send) {
            log.info("发送抽奖游戏IM消息-活动开始倒计时成功 {}", imMsgStr);
        } else {
            log.error("发送抽奖游戏IM消息-活动开始倒计时失败 ", imMsgStr);
        }
        return send;
    }

    /**
     * 保存奖项总数量到缓存
     *
     * @param snapId
     */
    private void saveRewardCountToCache(long snapId, List<LotteryGameRewardBO> rewards) {
        Integer count = rewards.stream()
                .filter(Objects::nonNull)
                .filter(reward -> reward.getRewardStock() != null)
                .map(LotteryGameRewardBO::getRewardStock)
                .reduce(Integer::sum)
                .orElse(0);
        String key = RedisActivityKeys.getLotteryGameRewardCount(snapId);
        redisService.set(key, "" + count);
        log.info("保存奖项库存总量到缓存 snapId={} count={} countByRedis={}", snapId, count, redisService.get(key));
    }

    /**
     * 保存所有奖品到缓存
     *
     * @param snapId
     * @param rewards
     */
    private void saveAllRewardsToCache(final long snapId, final List<LotteryGameRewardBO> rewards) {
        List<String> allRewards = rewards.stream()
                .filter(Objects::nonNull)
                .filter(reward -> reward.getRewardStock() != null)
                .filter(reward -> reward.getRewardId() != null)
                .filter(reward -> reward.getRewardType() != null)
                .map(reward -> {
                    String rewardKey = RedisActivityKeys.getLotteryGameWinItemsMapField(reward.getRewardType(), reward.getRewardId());
                    return IntStream.range(0, reward.getRewardStock())
                            .mapToObj(i -> rewardKey)
                            .collect(Collectors.toList());
                })
                .flatMap(rewardStream -> rewardStream.stream())
                .collect(Collectors.toList());
        Collections.shuffle(allRewards);
        final String key = RedisActivityKeys.getLotteryGameRewards(snapId);
        redisService.lLeftPushAll(key, allRewards);
    }

    /**
     * 保存抽奖游戏奖品信息摘要到缓存
     *
     * @param snapId
     * @param rewards
     */
    private void saveRewardSummariesToCache(long snapId, List<LotteryGameRewardBO> rewards,
                                            Map<Long, PropsVO> propsMap, Map<Long, GiftDO> giftMap) {
        List<LotteryGameRewardSummary> rewardSummaries = rewards.stream()
                .map(reward -> toRewardSummary(reward, giftMap, propsMap))
                .collect(Collectors.toList());
        String key = RedisActivityKeys.getLotteryGameRewardSummaries(snapId);
        redisService.set(key, JSON.toJSONString(rewardSummaries), 6, TimeUnit.DAYS);
    }

    /**
     * 保存奖品库存到缓存
     *
     * @param snapId
     * @param rewards
     */
    private void saveRewardsStockToCache(long snapId, List<LotteryGameRewardBO> rewards) {
        rewards.stream()
                .forEach(reward -> {
                    // rewardType:rewardId = rewardStock
                    String key = RedisActivityKeys.getLotteryGameRewardStock(snapId, reward.getRewardType(), reward.getRewardId());
                    redisService.set(key, reward.getRewardStock() + "", 6L, TimeUnit.HOURS);
                });
    }

    /**
     * 发送周年庆-瓜分蛋糕-配置礼物总价值消息
     *
     * @param snapId
     * @param rewards
     */
    private void sendSetupGiftTotalValueMsg(long snapId, List<LotteryGameRewardBO> rewards) {
        long totalValue = rewards.stream()
                .filter(Objects::nonNull)
                .filter(reward -> reward.getRewardTotalValue() != null)
                .mapToLong(LotteryGameRewardBO::getRewardTotalValue)
                .sum();
        QueueDivideCakeSetupGiftTotalValueMsg msg = new QueueDivideCakeSetupGiftTotalValueMsg();
        msg.setSnapId(snapId);
        msg.setGiftTotalValue(totalValue);
        rocketProducer.sendMsg(RocketTopic.TPC_NORMAL, RocketTag.DIVIDE_CAKE_SETUP_GIFT_TOTAL_VALUE, msg.serialize());
    }

    private void saveWinItemsInRedisMap(long snapId, List<LotteryGameRewardBO> rewards,
                                        final Map<Long, PropsVO> propsMap, final Map<Long, GiftDO> giftMap) {
        String key = RedisActivityKeys.getLotteryGameWinItemsMap(snapId);
        rewards.stream()
                .forEach(reward -> {
                    String field = RedisActivityKeys.getLotteryGameWinItemsMapField(reward.getRewardType(), reward.getRewardId());
                    LotteryGameWinItemBO winItem = new LotteryGameWinItemBO();
                    BeanUtils.copyProperties(reward, winItem);

                    LotteryGameRewardSummary summary = toRewardSummary(reward, giftMap, propsMap);
                    winItem.setRewardPrice(summary.getRewardValue());
                    redisService.hashPut(key, field, JSON.toJSONString(winItem));
                });

        Map<Object, Object> map = redisService.hashGetAll(key);
        log.info("Redis中抽奖游戏奖品Map snapId={} {}", snapId, LogUtil.obj2JsonObj(map));
    }

    private LotteryGameRewardSummary toRewardSummary(LotteryGameRewardBO reward,
                                                     Map<Long, GiftDO> giftMap, Map<Long, PropsVO> propsMap) {
        LotteryGameRewardSummary summary = new LotteryGameRewardSummary();
        BeanUtils.copyProperties(reward, summary);

        Long rewardId = reward.getRewardId();
        String rewardName = "";
        Long price = 0L;
        LotteryGameRewardTypeEnum type = LotteryGameRewardTypeEnum.toEnum(reward.getRewardType());
        switch (type) {
            case GIFT:
                GiftDO gift = giftMap.get(rewardId);
                if (gift != null) {
                    rewardName = gift.getGiftName();
                    price = gift.getGiftPrice();
                }
                break;
            case PROPS:
                PropsVO props = propsMap.get(rewardId);
                if (props != null) {
                    rewardName = props.getPropsName();
                    price = props.getPropsExPrice();
                }
                break;
        }
        summary.setRewardName(rewardName);
        summary.setRewardValue(price);
        return summary;
    }

    /**
     * 获得奖品id列表
     *
     * @param reward
     * @param typeEnum
     * @return
     */
    private List<Long> getRewardIds(List<LotteryGameRewardBO> reward, LotteryGameRewardTypeEnum typeEnum) {
        return reward.stream()
                .filter(Objects::nonNull)
                .filter(item -> LotteryGameRewardTypeEnum.toEnum(item.getRewardType()) == typeEnum)
                .map(item -> item.getRewardId())
                .filter(Objects::nonNull)
                .distinct()
                .collect(Collectors.toList());
    }

    /**
     * 保存正在进行中的抽奖游戏快照id到缓存
     *
     * @param snapId
     */
    private void saveSnapId2Cache(long snapId, long minEndTime, long maxEndTime) {
        redisService.hashPut(RedisActivityKeys.getLotteryGameInProgressingSnapIds(), snapId + "",
                String.format("%s,%s", minEndTime, maxEndTime));
        log.info("保存正在进行中的抽奖游戏快照id到缓存 snapId={} minEndTime={} maxEndTime={}",
                snapId, minEndTime, maxEndTime);
    }

    /**
     * 添加价格信息
     *
     * @param item
     * @param giftPriceMap
     * @param propsPriceMap
     * @return
     */
    private LotteryGameWinItemBO addPriceInfo(LotteryGameWinItemBO item, Map<Long, PriceInfoBO> giftPriceMap,
                                              Map<Long, PriceInfoBO> propsPriceMap) {
        LotteryGameRewardTypeEnum type = LotteryGameRewardTypeEnum.toEnum(item.getRewardType());
        if (type == null || type == LotteryGameRewardTypeEnum.NONE) {
            item.setRewardPrice(0L);
            return item;
        }
        Long id = item.getRewardId();
        PriceInfoBO priceInfo = type == LotteryGameRewardTypeEnum.GIFT ? giftPriceMap.get(id) : propsPriceMap.get(id);
        item.setRewardPrice(priceInfo.getPrice());
        return item;
    }
}
