package com.starbuds.server.service.activity.provider;

import cn.hutool.core.collection.CollectionUtil;
import cn.hutool.core.convert.Convert;
import com.alibaba.fastjson.JSON;
import com.starbuds.server.common.constant.cache.RedisGiftKeys;
import com.starbuds.server.common.pojo.activity.GiftRankItem;
import com.starbuds.server.common.pojo.core.Tuple;
import com.starbuds.server.common.pojo.data.HeadwearVO;
import com.starbuds.server.common.pojo.db.ActivityWeekDO;
import com.starbuds.server.common.pojo.db.GiftDO;
import com.starbuds.server.common.pojo.user.UserCardVO;
import com.starbuds.server.common.service.data.HeadwearService;
import com.starbuds.server.common.service.data.RedisService;
import com.starbuds.server.common.service.media.GiftService;
import com.starbuds.server.common.service.user.RtcUserService;
import com.starbuds.server.common.service.user.UserService;
import com.starbuds.server.service.activity.dao.ActivityWeekDAO;
import lombok.extern.slf4j.Slf4j;
import org.apache.dubbo.config.annotation.Reference;
import org.springframework.stereotype.Component;

import javax.annotation.Resource;
import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.*;
import java.util.function.Function;
import java.util.stream.Collectors;

/**
 * 排行服务
 *
 * @author Lay
 * @date 2020/7/14
 */
@Slf4j
@Component
public class GiftRankProvider {

    @Resource
    ActivityWeekDAO activityWeekDAO;

    @Reference(version = "1.0.0")
    RedisService redisService;

    @Reference(version = "1.0.0")
    UserService userService;

    @Reference(version = "1.0.0")
    GiftService giftService;

    @Reference(version = "1.0.0")
    RtcUserService rtcUserService;

    @Reference(version = "1.0.0")
    HeadwearService headwearService;

    static {
        StringBuilder sb = new StringBuilder();
        // 加日榜
        sb.append("redis.call('ZINCRBY', KEYS[1], ARGV[2], ARGV[1]);");
        // 周榜
        sb.append("redis.call('ZINCRBY', KEYS[2], ARGV[2], ARGV[1]);");
        // 月榜
        sb.append("redis.call('ZINCRBY', KEYS[3], ARGV[2], ARGV[1]);");
        LUA_INCR = sb.toString();
    }

    static {
        StringBuilder sb = new StringBuilder();
        sb.append("local t = redis.call('TIME');");
        sb.append("return t[1]");
        LUA_TIME = sb.toString();
    }

    /**
     * 中央时间戳
     */
    public static final String LUA_TIME;

    /**
     * 加榜
     */
    public static final String LUA_INCR;

    /**
     * 获取 Redis 时间戳
     *
     * @return
     */
    private long getRedisTime() {
        return redisService.luaLong(LUA_TIME, new ArrayList<>(), new ArrayList<>());
    }

    public String[] getRankTimes() {
        long redisTime = getRedisTime();
        String[] times = new String[3];

        // 日
        Date date = new Date(redisTime * 1000);
        SimpleDateFormat sdfDay = new SimpleDateFormat("yyyyMMdd");
        times[0] = sdfDay.format(date);

        // 基础周为 2020年1月6日第一周
        long baseTime = 1578240000;
        long secDiff = redisTime - baseTime;
        double dayDiff = secDiff / 3600 / 24;
        int week = (int) (dayDiff / 7 + 1);
        times[1] = String.valueOf(week);

        // 月
        SimpleDateFormat sdfMonth = new SimpleDateFormat("yyyyMM");
        times[2] = sdfMonth.format(date);
        log.info("本期数组 times {}", JSON.toJSONString(times));
        return times;
    }

    /**
     * 获取上期榜单
     *
     * @return
     */
    public String[] getPreviousRankTimes() {

        String[] previousArray = new String[3];

        // 昨日
        SimpleDateFormat sdfDay = new SimpleDateFormat("yyyyMMdd");
        Calendar curCal = Calendar.getInstance();
        curCal.add(Calendar.DAY_OF_YEAR, -1);
        previousArray[0] = Convert.toStr(sdfDay.format(curCal.getTime()));

        // 获取上期的活动
        ActivityWeekDO weekDO = activityWeekDAO.selectLast();

        if (weekDO == null) {
            previousArray[1] = "0";
        } else {
            // 开始时间
            Long weekStartDate = weekDO.getStartDate();

            // 基础周为 2020年1月6日第一周
            long baseTime = 1578240000;
            long secDiff = 0;
            try {
                secDiff = sdfDay.parse(Convert.toStr(weekStartDate)).getTime() / 1000 - baseTime;
            } catch (ParseException e) {
                log.info("出现错误,{}", e);
            }

            double dayDiff = secDiff / 3600 / 24;
            int week = (int) (dayDiff / 7 + 1);
            previousArray[1] = String.valueOf(week);
        }

        // 上月
        SimpleDateFormat sdfMonth = new SimpleDateFormat("yyyyMM");
        Calendar monthCal = Calendar.getInstance();
        monthCal.add(Calendar.MONTH, -1);
        previousArray[2] = Convert.toStr(sdfMonth.format(curCal.getTime()));

        log.info("上期数组 previousArray {}", JSON.toJSONString(previousArray));
        return previousArray;
    }

    /**
     * 获取周
     *
     * @param startDate
     * @return
     */
    public String getWeek(Long startDate) {
        SimpleDateFormat sdf = new SimpleDateFormat("yyyyMMdd");
        long baseTime = 1578240000;
        try {
            long secDiff = sdf.parse(Convert.toStr(startDate)).getTime() / 1000 - baseTime;
            double dayDiff = secDiff / 3600 / 24;
            int week = (int) (dayDiff / 7 + 1);
            return Convert.toStr(week);
        } catch (Exception e) {
            log.error("出现错误", e);
        }
        return "";
    }

    /**
     * 获取排行榜
     *
     * @param key
     * @param start
     * @param end
     * @param giftId
     * @return
     */
    public List<GiftRankItem> getRankItems(String key, long start, long end, Long giftId) {
        Set<Tuple> rankData = redisService.zReverseRangeWithScores(key, start, end);

        if (CollectionUtil.isNotEmpty(rankData)) {
            List<GiftRankItem> giftRankItems = new ArrayList<>();
            // 礼物
            GiftDO gift = giftService.getGift(giftId, null);

            // 需要加载的用户
            Set<Long> userIds = new HashSet<>();

            for (Tuple data : rankData) {
                Long userId = Convert.toLong(data.getElement());
                userIds.add(userId);
                GiftRankItem giftRankItem = new GiftRankItem();
                giftRankItem.setUserId(userId);
                giftRankItem.setQuantity(Convert.toLong(data.getScore()));
                giftRankItem.setGiftId(giftId);
                giftRankItem.setGiftName(Optional.ofNullable(gift).map(GiftDO::getGiftName).orElse(""));
                giftRankItem.setValue(giftRankItem.getQuantity() * Optional.ofNullable(gift).map(GiftDO::getGiftPrice).orElse(0L));
                giftRankItems.add(giftRankItem);
            }

            // 拉取用户数据
            List<UserCardVO> userCardVOList = userService.getNameAndAvatarByIds(new ArrayList<>(userIds));
            Map<Long, UserCardVO> userMap = userCardVOList.stream().collect(Collectors.toMap(UserCardVO::getUserId, Function.identity()));

            UserCardVO defaultUser = new UserCardVO();
            defaultUser.setUserName("已注销");
            // 排序 由大到小
            giftRankItems.sort(Comparator.comparingLong(GiftRankItem::getQuantity).reversed());
            int index = 0;
            // 房间
            Map<Long, Long> userRoomMap = rtcUserService.getUserRoomIdMap(userIds);
            // 头饰
            Map<Long, HeadwearVO> headwearMap = headwearService.getUsersEnabledHeadwear(new ArrayList<>(userIds));
            for (GiftRankItem giftRankItem : giftRankItems) {
                index++;
                UserCardVO userCardVO = userMap.getOrDefault(giftRankItem.getUserId(), defaultUser);
                if (userMap.containsKey(giftRankItem.getUserId())) {
                    giftRankItem.setUserName(userCardVO.getUserName());
                    giftRankItem.setUserAvatar(userCardVO.getUserAvatar());
                    giftRankItem.setWealthLevel(userCardVO.getWealthLevel());
                    giftRankItem.setIsOnline(userCardVO.getIsOnline());
                }
                giftRankItem.setRoomId(CollectionUtil.isNotEmpty(userRoomMap) ? userRoomMap.get(giftRankItem.getUserId()) : null);
                if (CollectionUtil.isNotEmpty(headwearMap)) {
                    HeadwearVO headwearVO = headwearMap.get(giftRankItem.getUserId());
                    giftRankItem.setAnimationFile(headwearVO != null ? headwearVO.getAnimationFile() : null);
                }
                giftRankItem.setRankNo(index);
            }

            return giftRankItems;
        }

        return Collections.emptyList();
    }

    /**
     * 添加周星榜
     *
     * @param giftId
     * @param receiveUserId
     * @param score
     */
    public void addWeekStarRankScore(Long giftId, Long receiveUserId, Long score) {
        if (score < 1L) {
            return;
        }

        String[] times = getRankTimes();

        // 加榜
        List<String> keys = new ArrayList<>();
        keys.add(RedisGiftKeys.getGiftRankWeekStarDay(giftId, times[0]));
        keys.add(RedisGiftKeys.getGiftRankWeekStarWeek(giftId, times[1]));
        keys.add(RedisGiftKeys.getGiftRankWeekStarMonth(giftId, times[2]));

        // 参数
        List<String> args = new ArrayList<>();
        args.add(String.valueOf(receiveUserId));
        args.add(String.valueOf(score));

        // 加榜
        redisService.luaLong(LUA_INCR, keys, args);

        if (giftId != 0) {
            addWeekStarRankScore(0L, receiveUserId, score);
        }
    }

    /**
     * 添加壕星榜
     *
     * @param giftId
     * @param senderUserId
     * @param score
     */
    public void addRichStarRankScore(Long giftId, Long senderUserId, Long score) {
        if (score < 1L) {
            return;
        }

        String[] times = getRankTimes();

        // 加榜
        List<String> keys = new ArrayList<>();
        keys.add(RedisGiftKeys.getGiftRankRichStarDay(giftId, times[0]));
        keys.add(RedisGiftKeys.getGiftRankRichStarWeek(giftId, times[1]));
        keys.add(RedisGiftKeys.getGiftRankRichStarMonth(giftId, times[2]));

        // 参数
        List<String> args = new ArrayList<>();
        args.add(String.valueOf(senderUserId));
        args.add(String.valueOf(score));

        // 加榜
        redisService.luaLong(LUA_INCR, keys, args);

        if (giftId != 0) {
            addRichStarRankScore(0L, senderUserId, score);
        }
    }

}
