package com.starbuds.server.service.activity.provider;

import com.alibaba.fastjson.JSON;
import com.google.common.base.Preconditions;
import com.starbuds.server.common.constant.activity.LotteryGameRewardTypeEnum;
import com.starbuds.server.common.pojo.activity.bo.LotteryGameRewardBO;
import com.starbuds.server.common.pojo.activity.bo.LotteryGameWinItemBO;
import com.starbuds.server.common.pojo.activity.qo.LotteryGameSnapshotPageListQO;
import com.starbuds.server.common.pojo.api.PageList;
import com.starbuds.server.common.pojo.daoplus.QueryWrapper;
import com.starbuds.server.common.pojo.daoplus.UpdateWrapper;
import com.starbuds.server.common.pojo.db.LotteryGameConfigDO;
import com.starbuds.server.common.pojo.db.LotteryGameConfigItemDO;
import com.starbuds.server.common.pojo.db.LotteryGameSnapshotDO;
import com.starbuds.server.common.pojo.db.LotteryGameTimerDO;
import com.starbuds.server.common.service.core.IdentityService;
import com.starbuds.server.service.activity.dao.LotteryGameDAO;
import com.starbuds.server.service.activity.dao.LotteryGameSnapshotDAO;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.collections.CollectionUtils;
import org.apache.commons.lang3.BooleanUtils;
import org.apache.commons.lang3.StringUtils;
import org.apache.dubbo.config.annotation.Reference;
import org.springframework.beans.BeanUtils;
import org.springframework.stereotype.Component;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.transaction.interceptor.TransactionAspectSupport;

import javax.annotation.Resource;
import java.util.*;
import java.util.stream.IntStream;

import static com.google.common.base.Preconditions.checkArgument;
import static com.google.common.base.Preconditions.checkNotNull;
import static com.starbuds.server.common.constant.activity.LotteryGameTimerStatusEnum.IN_PROGRESS;
import static com.starbuds.server.common.constant.activity.LotteryGameTimerStatusEnum.STANDBY;
import static com.starbuds.server.common.pojo.util.PageListUtil.buildPageListQueryWrapper;

/**
 * 抽奖游戏活动执行快照
 */
@Component
@Slf4j
public class LotteryGameSnapshotProvider {

    /**
     * 主键
     */
    private String PRIMARY_KEY = "snapId";

    @Reference(version = "1.0.0")
    private IdentityService identityService;

    @Resource
    private LotteryGameSnapshotDAO lotteryGameSnapshotDAO;

    @Resource
    private LotteryGameDAO lotteryGameDAO;

    @Resource
    private LotteryGameConfigItemProvider lotteryGameConfigItemProvider;

    private static final String ERR_CREATE_MSG = "创建抽奖游戏活动执行快照失败 ";

    /**
     * 创建抽奖游戏活动执行快照
     *
     * @param timerId(tb_lottery_game_timer)
     * @return null:插入失败
     */
    @Transactional(rollbackFor = Exception.class)
    public LotteryGameSnapshotDO createLotteryGameSnapshot(long timerId) {
        LotteryGameTimerDO timer = getTimer(timerId);
        LotteryGameConfigItemDO item = getItem(timer.getLotteryGameConfigItemId());
        LotteryGameConfigDO config = getConfig(item.getConfigId());

        LotteryGameSnapshotDO po = new LotteryGameSnapshotDO();
        BeanUtils.copyProperties(timer, po);
        BeanUtils.copyProperties(item, po);
        BeanUtils.copyProperties(config, po);

        Long countDown = item.getCountDown();
        // 发送倒计时通知时间
        Long sendNoticeTime = timer.getStartTime();
        // 用户参与互动时间
        Long startPlayTime = sendNoticeTime + (countDown * 1000);
        Long minEndTime = startPlayTime + (item.getMinDurationSeconds() * 1000);
        Long maxEndTime = startPlayTime + (item.getMaxDurationSeconds() * 1000);
        po.setStartTime(startPlayTime);
        po.setSendNoticeTime(sendNoticeTime);
        po.setMinEndTime(minEndTime);
        po.setMaxEndTime(maxEndTime);
        return create(po);
    }

    /**
     * 插入数据
     *
     * @param po
     * @return null:插入失败
     */
    @Transactional(rollbackFor = Exception.class)
    public LotteryGameSnapshotDO create(LotteryGameSnapshotDO po) {
        po.setSnapId(identityService.getId());
        po.setCreateTime(System.currentTimeMillis());
        boolean flag = lotteryGameSnapshotDAO.insert(po) == 1;

        String json = JSON.toJSONString(po);
        if (flag) {
            log.info("抽奖游戏活动执行快照创建成功 {}", json);
            return po;
        }
        log.warn("抽奖游戏活动执行快照创建失败 {}", json);
        return null;
    }

    /**
     * 插入批量数据
     */
    @Transactional(rollbackFor = Exception.class)
    public boolean createBatch(List<LotteryGameSnapshotDO> list) {
        boolean flag = lotteryGameSnapshotDAO.insertBatch(list) > 0;
        return flag;
    }

    /**
     * 插入或更新数据
     */
    @Transactional(rollbackFor = Exception.class)
    public boolean createOrUpdate(LotteryGameSnapshotDO po) {
        boolean flag = lotteryGameSnapshotDAO.insertOrUpdate(po) == 1;
        return flag;
    }

    /**
     * 更新数据
     */
    @Transactional(rollbackFor = Exception.class)
    public boolean updateById(LotteryGameSnapshotDO po) {
        po.setCreateTime(null);

        UpdateWrapper uw = new UpdateWrapper();
        uw.setPo(po);
        QueryWrapper qw = new QueryWrapper();
        qw.eq(PRIMARY_KEY, po.getSnapId());
        uw.setQw(qw);
        return update(uw);
    }

    /**
     * 更新数据
     */
    @Transactional(rollbackFor = Exception.class)
    public boolean update(UpdateWrapper uw) {
        boolean flag = lotteryGameSnapshotDAO.update(uw) > 0;
        return flag;
    }

    /**
     * 根据id查询
     */
    public LotteryGameSnapshotDO getById(Long id) {
        QueryWrapper qw = new QueryWrapper();
        qw.eq(PRIMARY_KEY, id);
        return lotteryGameSnapshotDAO.selectOne(qw);
    }

    /**
     * 判断是否存在
     *
     * @param id 主键
     * @return true:存在
     */
    public boolean exist(Long id) {
        if (id == null) {
            return false;
        }
        QueryWrapper qw = new QueryWrapper();
        qw.eq(PRIMARY_KEY, id);
        return lotteryGameSnapshotDAO.selectCount(qw) > 0;
    }

    /**
     * 分页查询
     */
    public PageList<LotteryGameSnapshotDO> getPageList(LotteryGameSnapshotPageListQO qo) {
        QueryWrapper qw = buildPageListQueryWrapper(qo);
        PageList<LotteryGameSnapshotDO> page = new PageList<>(qo.getPageIndex(), qo.getPageSize());
        qw.setPageCursor(page.getPageCursor());
        qw.setPageSize(page.getPageSize());
        page.setList(lotteryGameSnapshotDAO.selectPageList(qw));
        page.setTotal(lotteryGameSnapshotDAO.selectCount(qw));
        return page;
    }

    /**
     * 根据ids获得 DO Map
     *
     * @param ids id集合
     * @return Map<主键, DO对象>
     */
    public Map<Long, LotteryGameSnapshotDO> getMapByIds(Collection ids) {
        return lotteryGameSnapshotDAO.selectMapByIds(ids);
    }

    // 手动编码开始

    /**
     * 获得抽奖游戏奖品列表
     *
     * @param snapId
     * @return
     */
    public List<LotteryGameRewardBO> getRewards(long snapId) {
        LotteryGameSnapshotDO snap = lotteryGameSnapshotDAO.selectRewardPool(snapId);
        if (snap == null) {
            log.error("获得抽奖游戏奖品列表失败，snap is null snapId=" + snapId);
            return Collections.emptyList();
        }
        String rewardPool = snap.getRewardPool();
        Preconditions.checkState(StringUtils.isNotBlank(rewardPool), "获得抽奖游戏奖品列表失败，rewardPool is blank snapId=" + snapId);
        List<LotteryGameRewardBO> rewards = JSON.parseArray(rewardPool, LotteryGameRewardBO.class);
        return CollectionUtils.isEmpty(rewards) ? Collections.emptyList() : rewards;
    }

    /**
     * 抽奖活动开出的奖品列表
     *
     * @param snapId
     * @return
     */
    @Deprecated
    public List<LotteryGameWinItemBO> getLotteryGameRewards(long snapId) {
        List<LotteryGameRewardBO> rewards = getRewards(snapId);
        if (CollectionUtils.isEmpty(rewards)) {
            return Collections.emptyList();
        }

        List<LotteryGameWinItemBO> winItems = new ArrayList<>();
        rewards.stream()
                .filter(Objects::nonNull)
                .forEach(reward -> {
                    IntStream.range(0, reward.getRewardStock())
                            .forEach(i -> {
                                LotteryGameWinItemBO winItem = new LotteryGameWinItemBO();
                                BeanUtils.copyProperties(reward, winItem);
                                winItems.add(winItem);
                            });
                });
        Collections.shuffle(winItems);
        return winItems;
    }

    private LotteryGameTimerDO getTimer(long timerId) {
        LotteryGameTimerDO timer = lotteryGameDAO.selectLotteryGameTimer(timerId, STANDBY.value);
        checkNotNull(timer, ERR_CREATE_MSG +
                String.format("timer is null timerId=%s status=%s", timerId, STANDBY.value));
        Long itemId = timer.getLotteryGameConfigItemId();
        checkNotNull(itemId, ERR_CREATE_MSG +
                String.format("itemId is null timerId=%s status=%s", timerId, STANDBY.value));
        checkNotNull(timer.getStartTime(), ERR_CREATE_MSG +
                String.format("startTime is null timerId=%s status=%s", timerId, STANDBY.value));
        return timer;
    }

    private LotteryGameConfigItemDO getItem(Long itemId) {
        LotteryGameConfigItemDO item = lotteryGameConfigItemProvider.getById(itemId);
        checkNotNull(item, ERR_CREATE_MSG + String.format("item is null itemId=%s", itemId));
        Long configId = item.getConfigId();
        checkNotNull(configId, ERR_CREATE_MSG + String.format("configId is null configId=%s", configId));
        checkNotNull(item.getMinDurationSeconds(), ERR_CREATE_MSG +
                String.format("minDurationSeconds is null itemId=%s", itemId));
        checkNotNull(item.getMaxDurationSeconds(), ERR_CREATE_MSG +
                String.format("maxDurationSeconds is null itemId=%s", itemId));
        checkNotNull(item.getCountDown(), ERR_CREATE_MSG +
                String.format("countDown is null itemId=%s", itemId));
        checkNotNull(item.getClickRate(), ERR_CREATE_MSG +
                String.format("clickRate is null itemId=%s", itemId));
        checkNotNull(item.getIconUrl(), ERR_CREATE_MSG +
                String.format("iconUrl is null itemId=%s", itemId));

        String rewardPool = item.getRewardPool();

        checkArgument(StringUtils.isNotBlank(rewardPool), ERR_CREATE_MSG +
                String.format("rewardPool is Blank itemId=%s rewardPool=%s", itemId, rewardPool));
        List<LotteryGameRewardBO> rewards = JSON.parseArray(rewardPool, LotteryGameRewardBO.class);
        checkNotNull(rewards, ERR_CREATE_MSG +
                String.format("rewardPool 奖项为空 itemId=%s rewardPool=%s", itemId, rewardPool));
        checkArgument(rewards.stream().filter(Objects::isNull).count() == 0, ERR_CREATE_MSG +
                String.format("rewardPool 奖项中存在 null itemId=%s rewards=%s", itemId, JSON.toJSONString(rewards, true)));
        long errCount = rewards.stream()
                .map(reward -> {
                    LotteryGameRewardTypeEnum rewardType = LotteryGameRewardTypeEnum.toEnum(reward.getRewardType());
                    if (rewardType == null) {
                        log.error("rewardType is null");
                        return true;
                    }
                    Long rewardId = reward.getRewardId();
                    if (rewardType != LotteryGameRewardTypeEnum.NONE && (rewardId == null || rewardId < 1)) {
                        log.error("rewardId is invalid");
                        return true;
                    }
                    Integer rewardNum = reward.getRewardNum();
                    if (rewardNum == null || rewardNum < 1) {
                        log.error("rewardNum is invalid:" + rewardNum);
                        return true;
                    }
                    Integer rewardStock = reward.getRewardStock();
                    if (rewardStock == null || rewardStock < 1) {
                        log.error("rewardStock is invalid:" + rewardStock);
                        return true;
                    }
                    Long rewardTotalValue = reward.getRewardTotalValue();
                    return rewardTotalValue == null;
                })
                .filter(BooleanUtils::isTrue)
                .count();
        checkArgument(errCount == 0, ERR_CREATE_MSG +
                String.format("rewardPool 奖项中存在错误的值 configId=%s rewardPool=%s", configId, rewardPool));

        return item;
    }

    private LotteryGameConfigDO getConfig(Long configId) {
        LotteryGameConfigDO config = lotteryGameDAO.selectLotteryGameConfig(configId);
        checkNotNull(config, ERR_CREATE_MSG + String.format("config is null configId=%s", configId));
        return config;
    }
}