package com.starbuds.server.service.activity.provider;

import cn.hutool.core.collection.CollectionUtil;
import com.alibaba.fastjson.JSON;
import com.googlecode.aviator.AviatorEvaluator;
import com.starbuds.server.common.constant.activity.TriggerActionTypeEnum;
import com.starbuds.server.common.constant.activity.TriggerEventEnum;
import com.starbuds.server.common.pojo.activity.*;
import com.starbuds.server.common.pojo.db.ActivityTriggerDO;
import com.starbuds.server.common.pojo.finance.BagItem;
import com.starbuds.server.common.service.finance.BagService;
import com.starbuds.server.common.service.finance.WalletService;
import com.starbuds.server.common.service.user.ScoreService;
import com.starbuds.server.common.service.user.UserService;
import com.starbuds.server.service.activity.constant.AsyncThreadConstant;
import com.starbuds.server.service.activity.dao.ActivityTriggerDAO;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.apache.dubbo.config.annotation.Reference;
import org.springframework.scheduling.annotation.Async;
import org.springframework.stereotype.Component;

import javax.annotation.Resource;
import java.util.*;

/**
 * 备注
 *
 * @author Lay
 * @date 2020/1/13
 */
@Component
@Slf4j
public class TriggerProvider {

    @Reference(version = "1.0.0")
    BagService bagService;

    @Reference(version = "1.0.0")
    WalletService walletService;

    @Reference(version = "1.0.0")
    ScoreService scoreService;

    @Reference(version = "1.0.0")
    UserService userService;

    @Resource
    ActivityTriggerDAO activityTriggerDAO;

    /**
     * 异步事件触发处理
     *
     * @param triggerEvent
     * @param eventParam
     * @return
     */
    @Async(AsyncThreadConstant.DEFAULT)
    public void handleEventAsync(TriggerEventEnum triggerEvent, BaseEventParam eventParam) {
        log.info("触发事件 {}", triggerEvent.getDesc());
        switch (triggerEvent) {
            case USER_REG:
                handleUserRegEvent((UserRegEventParam) eventParam);
                break;
            case USER_LOGIN:
                handleUserLoginEvent((UserLoginEventParam) eventParam);
                break;
            case USER_RECHARGE:
                handleUserRecharge((UserRechargeEventParam) eventParam);
                break;
            case NOBLE_NEW:
                handleNobleNew((NobleNewEventParam) eventParam);
                break;
            case NOBLE_RENEW:
                handleNobleRenew((NobleRenewEventParam) eventParam);
                break;
            case NOBLE_UPGRADE:
                handleNobleUpgrade((NobleUpgradeEventParam) eventParam);
                break;
            case SCORE_LEVEL_UP:
                handleScoreLevelUp((ScoreLevelUpEventParam) eventParam);
                break;
            case VIP_NEW:
                handleVipNew((VipNewEventParam) eventParam);
                break;
            case VIP_RENEW:
                handleVipRenew((VipRenewEventParam) eventParam);
                break;
            case RTC_LOVE_MATCH:
                handleRtcLoveMatch((RtcLoveMatchEventParam) eventParam);
                break;
            default:
                break;
        }
    }

    /**
     * 处理用户注册事件
     *
     * @param eventParam
     */
    private void handleUserRegEvent(UserRegEventParam eventParam) {
        // 条件数据准备
        Map<String, Object> env = new HashMap<>();
        env.put("渠道", eventParam.getChannelId());
        executeWithConditions(TriggerEventEnum.USER_REG, env, eventParam.getUserId());
    }

    /**
     * 处理用户登录事件
     *
     * @param eventParam
     */
    private void handleUserLoginEvent(UserLoginEventParam eventParam) {
        // 条件数据准备
        Map<String, Object> env = new HashMap<>();
        env.put("渠道", eventParam.getChannelId());
        env.put("登录方式", eventParam.getLoginType());
        env.put("首次登录", eventParam.getFirstLogin());
        executeWithConditions(TriggerEventEnum.USER_LOGIN, env, eventParam.getUserId());
    }

    /**
     * 处理用户充值事件
     *
     * @param eventParam
     */
    private void handleUserRecharge(UserRechargeEventParam eventParam) {
        // 条件数据准备
        Map<String, Object> env = new HashMap<>();
        env.put("支付方式", eventParam.getPayWay());
        env.put("本次充值金额", eventParam.getAmount());
        env.put("今日充值金额", eventParam.getTodayAmount());
        env.put("首次充值", eventParam.getIsFirst());
        env.put("今日首充", eventParam.getIsTodayFirst());
        env.put("注册当日", eventParam.getIsRegDay());
        executeWithConditions(TriggerEventEnum.USER_RECHARGE, env, eventParam.getUserId());
    }

    /**
     * 处理开通贵族
     *
     * @param eventParam
     */
    private void handleNobleNew(NobleNewEventParam eventParam) {
        // 条件数据准备
        Map<String, Object> env = new HashMap<>();
        env.put("贵族等级", eventParam.getNobleLevel());
        env.put("开通时长", eventParam.getDuration());
        env.put("使用道具", eventParam.getIsFromProps());
        env.put("道具编号", eventParam.getPropsId());
        executeWithConditions(TriggerEventEnum.NOBLE_NEW, env, eventParam.getUserId());
    }

    /**
     * 处理续费贵族
     *
     * @param eventParam
     */
    private void handleNobleRenew(NobleRenewEventParam eventParam) {
        // 条件数据准备
        Map<String, Object> env = new HashMap<>();
        env.put("贵族等级", eventParam.getNobleLevel());
        env.put("续费时长", eventParam.getDuration());
        env.put("续费次数", eventParam.getTimes());
        executeWithConditions(TriggerEventEnum.NOBLE_RENEW, env, eventParam.getUserId());
    }

    /**
     * 处理升级贵族
     *
     * @param eventParam
     */
    private void handleNobleUpgrade(NobleUpgradeEventParam eventParam) {
        // 条件数据准备
        Map<String, Object> env = new HashMap<>();
        env.put("原贵族等级", eventParam.getOldNobleLevel());
        env.put("新贵族等级", eventParam.getNewNobleLevel());
        executeWithConditions(TriggerEventEnum.NOBLE_UPGRADE, env, eventParam.getUserId());
    }

    /**
     * 处理积分升级
     *
     * @param eventParam
     */
    private void handleScoreLevelUp(ScoreLevelUpEventParam eventParam) {
        // 条件数据准备
        Map<String, Object> env = new HashMap<>();
        env.put("类型", eventParam.getScoreType());
        env.put("等级", eventParam.getScoreLevel());
        env.put("积分", eventParam.getScoreValue());
        executeWithConditions(TriggerEventEnum.SCORE_LEVEL_UP, env, eventParam.getUserId());
    }

    /**
     * 处理开通会员
     *
     * @param eventParam
     */
    private void handleVipNew(VipNewEventParam eventParam) {
        // 条件数据准备
        Map<String, Object> env = new HashMap<>();
        env.put("开通天数", eventParam.getVipDays());
        env.put("会员等级", eventParam.getVipLevel());
        executeWithConditions(TriggerEventEnum.VIP_NEW, env, eventParam.getUserId());
    }

    /**
     * 处理续费会员
     *
     * @param eventParam
     */
    private void handleVipRenew(VipRenewEventParam eventParam) {
        // 条件数据准备
        Map<String, Object> env = new HashMap<>();
        env.put("续费天数", eventParam.getVipDays());
        env.put("会员等级", eventParam.getVipLevel());
        executeWithConditions(TriggerEventEnum.VIP_RENEW, env, eventParam.getUserId());
    }

    private void handleRtcLoveMatch(RtcLoveMatchEventParam eventParam) {
        // 条件数据准备
        Map<String, Object> env = new HashMap<>();
        env.put("心动值和", eventParam.getTotalScore());
        executeWithConditions(TriggerEventEnum.RTC_LOVE_MATCH, env, Arrays.asList(eventParam.getRedUserId(), eventParam.getBlueUserId()));
    }

    /**
     * 通过条件处理
     *
     * @param triggerEvent
     * @param envMap
     * @param userId
     */
    private void executeWithConditions(TriggerEventEnum triggerEvent, Map<String, Object> envMap, Long userId) {
        executeWithConditions(triggerEvent, envMap, Collections.singletonList(userId));
    }

    private void executeWithConditions(TriggerEventEnum triggerEvent, Map<String, Object> envMap, List<Long> userIds) {
        List<ActivityTriggerDO> activityTriggerList = activityTriggerDAO.selectByEvent(triggerEvent.getValue(), System.currentTimeMillis());
        if (CollectionUtil.isNotEmpty(activityTriggerList)) {
            // 条件数据准备
            for (ActivityTriggerDO activityTrigger : activityTriggerList) {
                if (StringUtils.isNotBlank(activityTrigger.getTriggerActions())) {
                    if (envMap != null) {
                        // 判断条件
                        String conditions = activityTrigger.getTriggerConditions();
                        log.info("conditions: {} , envMap：{}", conditions, envMap);
                        if (StringUtils.isNotBlank(conditions)) {
                            try {
                                boolean isConditionMatch = (boolean) AviatorEvaluator.execute(conditions, envMap);
                                if (!isConditionMatch) {
                                    continue;
                                }
                            } catch (Exception e) {
                                log.error("条件表达式执行失败", e);
                                // 执行出错时，不应该触发
                                continue;
                            }
                        }
                    }

                    executeEventActions(activityTrigger, userIds);
                }
            }
        }
    }

    /**
     * 执行触发器动作
     *
     * @param activityTrigger
     * @param userIds
     */
    private void executeEventActions(ActivityTriggerDO activityTrigger, List<Long> userIds) {
        // 转换触发器动作
        List<ActivityTrigger.Action> actions = JSON.parseArray(activityTrigger.getTriggerActions(), ActivityTrigger.Action.class);
        log.info("userIds：{},actions：{}", userIds, JSON.toJSONString(actions));
        // 动作备注
        String remarks = String.format("%s赠送", activityTrigger.getTriggerName());
        if (CollectionUtil.isNotEmpty(actions)) {
            for (Long userId : userIds) {
                for (ActivityTrigger.Action action : actions) {
                    TriggerActionTypeEnum triggerActionType = TriggerActionTypeEnum.toEnum(action.getActionType());
                    switch (triggerActionType) {
                        case GIVE_ITEM:
                            giveItem(activityTrigger.getTriggerId(), userId, action.getBagItems(), remarks);
                            break;
                        case GIVE_COIN:
                            giveCoin(activityTrigger.getTriggerId(), userId, action.getWalletType(), action.getWalletAmount(), remarks);
                            break;
                        case GIVE_SCORE:
                            giveScore(activityTrigger.getTriggerId(), userId, action.getScoreType(), action.getScoreValue(), remarks);
                            break;
                        default:
                            break;
                    }
                }
            }
        }
    }

    /**
     * 赠送物品
     *
     * @param triggerId
     * @param userId
     * @param items
     */
    private void giveItem(Long triggerId, Long userId, List<BagItem> items, String remarks) {
        for (BagItem bagItem : items) {
            bagService.addItem(userId, bagItem.getItemId(), bagItem.getItemType(), bagItem.getQuantity(), bagItem.getExpireTime().equals(-1L) ? -1L : System.currentTimeMillis() + bagItem.getExpireTime(), remarks, triggerId);
        }
    }

    /**
     * 赠送金币
     *
     * @param triggerId
     * @param userId
     * @param walletType
     * @param amount
     */
    private void giveCoin(Long triggerId, Long userId, Integer walletType, Long amount, String remarks) {
        walletService.addCoin(userId, walletType, amount, remarks, triggerId);
    }

    /**
     * 赠送积分
     *
     * @param triggerId
     * @param userId
     * @param scoreType
     * @param scoreValue
     */
    private void giveScore(Long triggerId, Long userId, Integer scoreType, Long scoreValue, String remarks) {
        scoreService.addScore(userId, scoreType, scoreValue, remarks, triggerId, 0L);
    }
}
