package com.starbuds.server.service.activity.rocket.handler;

import cn.hutool.core.collection.CollectionUtil;
import cn.hutool.core.util.RandomUtil;
import com.alibaba.fastjson.JSON;
import com.starbuds.server.common.constant.activity.ActivityHeadlineLotteryPeriodStatusEnum;
import com.starbuds.server.common.constant.activity.LotteryRewardTypeEnum;
import com.starbuds.server.common.constant.cache.RedisActivityKeys;
import com.starbuds.server.common.constant.finance.BagItemTypeEnum;
import com.starbuds.server.common.constant.message.im.ContentType;
import com.starbuds.server.common.constant.message.im.ImTargetType;
import com.starbuds.server.common.constant.statistics.StatPlatformTypeEnum;
import com.starbuds.server.common.pojo.activity.ActivityHeadlinePeriod;
import com.starbuds.server.common.pojo.activity.HeadlineLotteryRewardItem;
import com.starbuds.server.common.pojo.db.ActivityHeadlinePeriodDO;
import com.starbuds.server.common.pojo.db.ActivityHeadlinePeriodItemDO;
import com.starbuds.server.common.pojo.message.im.activity.HeadlineLotteryPeriodStatusMsg;
import com.starbuds.server.common.pojo.message.im.biz.SystemMsg;
import com.starbuds.server.common.pojo.mq.QueueHeadlineLotteryPeriodOverMsg;
import com.starbuds.server.common.service.data.PropsService;
import com.starbuds.server.common.service.data.RedisService;
import com.starbuds.server.common.service.finance.BagService;
import com.starbuds.server.common.service.media.ChatService;
import com.starbuds.server.common.service.media.GiftService;
import com.starbuds.server.common.service.message.ImService;
import com.starbuds.server.common.service.statistics.StatPlatformService;
import com.starbuds.server.common.service.user.UserService;
import com.starbuds.server.service.activity.dao.ActivityHeadlinePeriodDAO;
import com.starbuds.server.service.activity.dao.ActivityHeadlinePeriodItemDAO;
import com.starbuds.server.service.activity.headline.HeadlineLotteryHeatProvider;
import lombok.extern.slf4j.Slf4j;
import org.apache.dubbo.config.annotation.Reference;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Component;
import org.springframework.transaction.annotation.Transactional;

import javax.annotation.Resource;
import java.math.BigDecimal;
import java.util.Comparator;
import java.util.List;
import java.util.Optional;

/**
 * 延迟消息处理
 *
 * @author Lay
 * @date 2020/12/28
 */
@Component
@Slf4j
public class DelayMsgHandler {

    @Reference(version = "1.0.0")
    RedisService redisService;

    @Reference(version = "1.0.0")
    BagService bagService;

    @Reference(version = "1.0.0")
    UserService userService;

    @Reference(version = "1.0.0")
    ImService imService;

    @Reference(version = "1.0.0")
    StatPlatformService statPlatformService;

    @Reference(version = "1.0.0")
    GiftService giftService;

    @Reference(version = "1.0.0")
    PropsService propsService;

    @Reference(version = "1.0.0")
    ChatService chatService;

    @Resource
    ActivityHeadlinePeriodDAO activityHeadlinePeriodDAO;

    @Resource
    ActivityHeadlinePeriodItemDAO activityHeadlinePeriodItemDAO;

    @Resource
    HeadlineLotteryHeatProvider headlineLotteryHeatProvider;

    @Value("${name.diamond}")
    private String nameDiamond;

    /**
     * 头条抽奖活动结束
     *
     * @param msg
     */
    public void handleHeadlineLotteryPeriodOver(QueueHeadlineLotteryPeriodOverMsg msg) {
        long configId = msg.getConfigId();
        int periodId = msg.getPeriodId();
        ActivityHeadlinePeriod period = activityHeadlinePeriodDAO.selectPeriodById(configId, periodId);
        if (period == null) {
            log.error("未找到活动期数");
            return;
        }

        // 删除 key
        redisService.delKey(RedisActivityKeys.ACTIVITY_HEADLINE_LOTTERY_GOING);

        // 读取热度
        long heat = headlineLotteryHeatProvider.get(configId, periodId);
        log.info("热度 {}", heat);
        // 移除热度
        headlineLotteryHeatProvider.remove(configId, periodId);

        // 统计参数人数
        int usersCount = activityHeadlinePeriodItemDAO.countUsers(configId, periodId);

        // 统计参与次数
        int timesCount = activityHeadlinePeriodItemDAO.countTimes(configId, periodId);

        // 抽取幸运用户
        log.info("抽取幸运头条");
        ActivityHeadlinePeriodItemDO winItem = null;
        if (timesCount > 0) {
            int randomNum = RandomUtil.randomInt(0, timesCount);
            winItem = activityHeadlinePeriodItemDAO.selectItemByIndex(configId, periodId, randomNum);
            log.info("幸运头条 {}", winItem.getItemId());
        }

        // 确定奖项
        log.info("确定奖项");
        HeadlineLotteryRewardItem winReward = null;
        List<HeadlineLotteryRewardItem> rewardItems = JSON.parseArray(period.getPeriodRewardConfig(), HeadlineLotteryRewardItem.class);
        if (CollectionUtil.isNotEmpty(rewardItems)) {
            // 奖项排序
            rewardItems.sort(Comparator.comparingInt(HeadlineLotteryRewardItem::getMinHeat).reversed());
            for (HeadlineLotteryRewardItem rewardItem : rewardItems) {
                // 判断是否满足条件
                if (heat >= rewardItem.getMinHeat()) {
                    winReward = rewardItem;
                    break;
                }
            }
            log.info("奖项 {}", JSON.toJSONString(winReward));
        }

        // 更新活动
        ActivityHeadlinePeriodDO periodDO = new ActivityHeadlinePeriodDO();
        BeanUtils.copyProperties(period, periodDO);
        periodDO.setUsersCount(usersCount);
        periodDO.setTimesCount(timesCount);
        periodDO.setMaxHeat(heat);
        periodDO.setPeriodStatus(ActivityHeadlineLotteryPeriodStatusEnum.Over.getValue());
        if (winItem != null) {
            periodDO.setWinnerItemId(winItem.getItemId());
        }
        if (winReward != null) {
            periodDO.setWinReward(JSON.toJSONString(winReward));
        }
        int updated = activityHeadlinePeriodDAO.updateAfterOver(periodDO, ActivityHeadlineLotteryPeriodStatusEnum.Going.getValue());
        if (updated == 0) {
            log.info("重复处理");
            return;
        }

        // 发放奖励
        if (winItem != null && winReward != null) {
            if (LotteryRewardTypeEnum.GIFT.getValue() == winReward.getRewardType()) {
                // 发放礼物
                bagService.addItem(winItem.getUserId(), winReward.getRewardId(), BagItemTypeEnum.GIFT.getValue(), winReward.getQuantity(), null, "头条抽奖活动中奖", winItem.getItemId());
            } else if (LotteryRewardTypeEnum.PROPS.getValue() == winReward.getRewardType()) {
                // 发放道具
                bagService.addItem(winItem.getUserId(), winReward.getRewardId(), BagItemTypeEnum.PROPS.getValue(), winReward.getQuantity(), null, "头条抽奖活动中奖", winItem.getItemId());
            }
            log.info("用户ID {} , 礼物ID {}, 发放完成", winItem.getUserId(), winReward.getRewardId());

            // 增加活动中奖价值
            statPlatformService.addPlatformStat(0L, StatPlatformTypeEnum.HEADLINE_LOTTERY_ACTIVITY_WIN_AMOUNT.getValue(), BigDecimal.valueOf(winReward.getRewardValue()));

            // 发送通知
            SystemMsg systemMsg = new SystemMsg();
            systemMsg.setContentType(ContentType.TEXT);
            systemMsg.setBody(String.format("哇偶，幸运儿！恭喜您在%s第%d期活动中获得%sx%d，价值%d%s。奖励已发放至您的背包，请注意查收哦！",
                    period.getActivityName(), period.getPeriodId(), winReward.getRewardName(),
                    winReward.getQuantity(), winReward.getRewardValue(), nameDiamond));
            imService.sendCustomImMsg(null, winItem.getUserId(), systemMsg, null);
        }

        // 发布头条
        if (winItem != null) {
            chatService.sendHeadline(winItem.getUserId(), winItem.getContent(), true);
        }

        // 广播通知
        HeadlineLotteryPeriodStatusMsg periodStatusMsg = new HeadlineLotteryPeriodStatusMsg();
        periodStatusMsg.setConfigId(configId);
        periodStatusMsg.setPeriodId(periodId);
        periodStatusMsg.setPeriodStatus(ActivityHeadlineLotteryPeriodStatusEnum.Over.getValue());
        periodStatusMsg.setActivityName(period.getActivityName());
        periodStatusMsg.setWinner(Optional.ofNullable(winItem).map(t -> userService.getCard(t.getUserId())).orElse(null));
        periodStatusMsg.setRewardItem(winReward);
        periodStatusMsg.setHeadlineContent(Optional.ofNullable(winItem).map(ActivityHeadlinePeriodItemDO::getContent).orElse(null));
        imService.sendRtcRoomMsg(ImTargetType.GROUP, null, null, 0L, periodStatusMsg);
    }
}
